@@ -21,6 +21,8 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include "config.h"
+
 #include <sys/wait.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -38,6 +40,8 @@
 #include "shared/config-parser.h"
 #include "shared/helpers.h"
 #include "shared/os-compatibility.h"
+#include "shared/xalloc.h"
+#include "shared/zalloc.h"
 #include "ivi-application-client-protocol.h"
 #include "ivi-hmi-controller-client-protocol.h"
 
@@ -156,28 +160,10 @@ hmi_homescreen_setting {
 	char		*cursor_theme;
 	int32_t		cursor_size;
 	uint32_t	transition_duration;
+	uint32_t	surface_id_offset;
+	int32_t		screen_num;
 };
 
-static void *
-fail_on_null(void *p, size_t size, char *file, int32_t line)
-{
-	if (size && !p) {
-		fprintf(stderr, "%s(%d) %zd: out of memory\n",
-			file, line, size);
-		exit(EXIT_FAILURE);
-	}
-
-	return p;
-}
-
-static void *
-mem_alloc(size_t size, char *file, int32_t line)
-{
-	return fail_on_null(calloc(1, size), size, file, line);
-}
-
-#define MEM_ALLOC(s) mem_alloc((s),__FILE__,__LINE__)
-
 /*****************************************************************************
  *  Event Handler
  ****************************************************************************/
@@ -605,6 +591,10 @@ registry_handle_global(void *data, struct wl_registry *registry, uint32_t name,
 			wl_registry_bind(registry, name, &wl_shm_interface, 1);
 		wl_shm_add_listener(p_wlCtx->wlShm, &shm_listenter, p_wlCtx);
 	} else if (!strcmp(interface, "wl_seat")) {
+		/* XXX: should be handling multiple wl_seats */
+		if (p_wlCtx->wlSeat)
+			return;
+
 		p_wlCtx->wlSeat =
 			wl_registry_bind(registry, name, &wl_seat_interface, 1);
 		wl_seat_add_listener(p_wlCtx->wlSeat, &seat_Listener, data);
@@ -619,6 +609,8 @@ registry_handle_global(void *data, struct wl_registry *registry, uint32_t name,
 
 		ivi_hmi_controller_add_listener(p_wlCtx->hmiCtrl,
 				&hmi_controller_listener, p_wlCtx);
+	} else if (!strcmp(interface, "wl_output")) {
+		p_wlCtx->hmi_setting->screen_num++;
 	}
 }
 
@@ -762,7 +754,7 @@ create_cursors(struct wlContextCommon *cmm)
 						 cmm->wlShm);
 
 	cmm->cursors =
-		MEM_ALLOC(ARRAY_LENGTH(cursors) * sizeof(cmm->cursors[0]));
+		xzalloc(ARRAY_LENGTH(cursors) * sizeof(cmm->cursors[0]));
 
 	for (i = 0; i < ARRAY_LENGTH(cursors); i++) {
 		cursor = NULL;
@@ -1038,7 +1030,7 @@ create_launchers(struct wlContextCommon *cmm, struct wl_list *launcher_list)
 	if (0 == launcher_count)
 		return;
 
-	launchers = MEM_ALLOC(launcher_count * sizeof(*launchers));
+	launchers = xzalloc(launcher_count * sizeof(*launchers));
 
 	wl_list_for_each(launcher, launcher_list, link) {
 		launchers[ii] = launcher;
@@ -1056,7 +1048,7 @@ create_launchers(struct wlContextCommon *cmm, struct wl_list *launcher_list)
 		for (jj = start; jj <= ii; jj++) {
 			struct wlContextStruct *p_wlCtx;
 
-			p_wlCtx = MEM_ALLOC(sizeof(*p_wlCtx));
+			p_wlCtx = xzalloc(sizeof(*p_wlCtx));
 			p_wlCtx->cmm = cmm;
 			create_ivisurfaceFromFile(p_wlCtx,
 						  launchers[jj]->icon_surface_id,
@@ -1078,7 +1070,7 @@ hmi_homescreen_setting_create(void)
 	const char *config_file;
 	struct weston_config *config = NULL;
 	struct weston_config_section *shellSection = NULL;
-	struct hmi_homescreen_setting *setting = MEM_ALLOC(sizeof(*setting));
+	struct hmi_homescreen_setting *setting = xzalloc(sizeof(*setting));
 	struct weston_config_section *section = NULL;
 	const char *name = NULL;
 	uint32_t workspace_layer_id;
@@ -1159,6 +1151,9 @@ hmi_homescreen_setting_create(void)
 		shellSection, "workspace-background-id",
 		&setting->workspace_background.id, 2001);
 
+	weston_config_section_get_uint(
+		shellSection, "surface-id-offset", &setting->surface_id_offset, 10);
+
 	icon_surface_id = workspace_layer_id + 1;
 
 	while (weston_config_next_section(config, &section, &name)) {
@@ -1167,7 +1162,7 @@ hmi_homescreen_setting_create(void)
 		if (strcmp(name, "ivi-launcher") != 0)
 			continue;
 
-		launcher = MEM_ALLOC(sizeof(*launcher));
+		launcher = xzalloc(sizeof(*launcher));
 		wl_list_init(&launcher->link);
 
 		weston_config_section_get_string(section, "icon",
@@ -1199,8 +1194,8 @@ hmi_homescreen_setting_create(void)
 int main(int argc, char **argv)
 {
 	struct wlContextCommon wlCtxCommon;
-	struct wlContextStruct wlCtx_BackGround;
-	struct wlContextStruct wlCtx_Panel;
+	struct wlContextStruct *wlCtx_BackGround;
+	struct wlContextStruct *wlCtx_Panel;
 	struct wlContextStruct wlCtx_Button_1;
 	struct wlContextStruct wlCtx_Button_2;
 	struct wlContextStruct wlCtx_Button_3;
@@ -1211,12 +1206,11 @@ int main(int argc, char **argv)
 	int ret = 0;
 	struct hmi_homescreen_setting *hmi_setting;
 	struct wlContextStruct *pWlCtxSt = NULL;
+	int i = 0;
 
 	hmi_setting = hmi_homescreen_setting_create();
 
 	memset(&wlCtxCommon, 0x00, sizeof(wlCtxCommon));
-	memset(&wlCtx_BackGround, 0x00, sizeof(wlCtx_BackGround));
-	memset(&wlCtx_Panel,	  0x00, sizeof(wlCtx_Panel));
 	memset(&wlCtx_Button_1,   0x00, sizeof(wlCtx_Button_1));
 	memset(&wlCtx_Button_2,   0x00, sizeof(wlCtx_Button_2));
 	memset(&wlCtx_Button_3,   0x00, sizeof(wlCtx_Button_3));
@@ -1254,6 +1248,9 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
+	wlCtx_BackGround = xzalloc(hmi_setting->screen_num * sizeof(struct wlContextStruct));
+	wlCtx_Panel= xzalloc(hmi_setting->screen_num * sizeof(struct wlContextStruct));
+
 	if (wlCtxCommon.hmi_setting->cursor_theme) {
 		create_cursors(&wlCtxCommon);
 
@@ -1263,8 +1260,6 @@ int main(int argc, char **argv)
 		wlCtxCommon.current_cursor = CURSOR_LEFT_PTR;
 	}
 
-	wlCtx_BackGround.cmm = &wlCtxCommon;
-	wlCtx_Panel.cmm      = &wlCtxCommon;
 	wlCtx_Button_1.cmm   = &wlCtxCommon;
 	wlCtx_Button_2.cmm   = &wlCtxCommon;
 	wlCtx_Button_3.cmm   = &wlCtxCommon;
@@ -1273,11 +1268,18 @@ int main(int argc, char **argv)
 	wlCtx_WorkSpaceBackGround.cmm = &wlCtxCommon;
 
 	/* create desktop widgets */
-	create_background(&wlCtx_BackGround, hmi_setting->background.id,
-			  hmi_setting->background.filePath);
+	for (i = 0; i < hmi_setting->screen_num; i++) {
+		wlCtx_BackGround[i].cmm = &wlCtxCommon;
+		create_background(&wlCtx_BackGround[i],
+				  hmi_setting->background.id +
+					(i * hmi_setting->surface_id_offset),
+				  hmi_setting->background.filePath);
 
-	create_panel(&wlCtx_Panel, hmi_setting->panel.id,
-		     hmi_setting->panel.filePath);
+		wlCtx_Panel[i].cmm = &wlCtxCommon;
+		create_panel(&wlCtx_Panel[i],
+			     hmi_setting->panel.id + (i * hmi_setting->surface_id_offset),
+			     hmi_setting->panel.filePath);
+	}
 
 	create_button(&wlCtx_Button_1, hmi_setting->tiling.id,
 		      hmi_setting->tiling.filePath, 0);
@@ -1308,6 +1310,9 @@ int main(int argc, char **argv)
 		destroyWLContextStruct(pWlCtxSt);
 	}
 
+	free(wlCtx_BackGround);
+	free(wlCtx_Panel);
+
 	destroyWLContextCommon(&wlCtxCommon);
 
 	return 0;
