@@ -3,23 +3,24 @@
  * Copyright © 2010 Intel Corporation
  * Copyright © 2014 Collabora, Ltd.
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting documentation, and
- * that the name of the copyright holders not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  The copyright holders make no representations
- * about the suitability of this software for any purpose.  It is provided "as
- * is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
- * OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #include <config.h>
@@ -35,10 +36,10 @@
 #include <time.h>
 
 #include <wayland-client.h>
-#include "../shared/os-compatibility.h"
-#include "presentation_timing-client-protocol.h"
-
-#define ARRAY_LENGTH(a) (sizeof (a) / sizeof (a)[0])
+#include "shared/helpers.h"
+#include "shared/zalloc.h"
+#include "shared/os-compatibility.h"
+#include "presentation-time-client-protocol.h"
 
 enum run_mode {
 	RUN_MODE_FEEDBACK,
@@ -67,7 +68,7 @@ struct display {
 	struct wl_shm *shm;
 	uint32_t formats;
 
-	struct presentation *presentation;
+	struct wp_presentation *presentation;
 	clockid_t clk_id;
 
 	struct wl_list output_list; /* struct output::link */
@@ -76,7 +77,7 @@ struct display {
 struct feedback {
 	struct window *window;
 	unsigned frame_no;
-	struct presentation_feedback *feedback;
+	struct wp_presentation_feedback *feedback;
 	struct timespec commit;
 	struct timespec target;
 	uint32_t frame_stamp;
@@ -101,6 +102,7 @@ struct window {
 	int num_buffers;
 	int next;
 	int refresh_nsec;
+	int commit_delay_msecs;
 
 	struct wl_callback *callback;
 	struct wl_list feedback_list;
@@ -201,19 +203,21 @@ static const struct wl_shell_surface_listener shell_surface_listener = {
 
 static struct window *
 create_window(struct display *display, int width, int height,
-	      enum run_mode mode)
+	      enum run_mode mode, int commit_delay_msecs)
 {
 	struct window *window;
 	char title[128];
 	int ret;
 
 	snprintf(title, sizeof(title),
-		 "presentation-shm: %s", run_mode_name[mode]);
+		 "presentation-shm: %s [Delay %i msecs]", run_mode_name[mode],
+		 commit_delay_msecs);
 
-	window = calloc(1, sizeof *window);
+	window = zalloc(sizeof *window);
 	if (!window)
 		return NULL;
 
+	window->commit_delay_msecs = commit_delay_msecs;
 	window->mode = mode;
 	window->callback = NULL;
 	wl_list_init(&window->feedback_list);
@@ -248,7 +252,7 @@ static void
 destroy_feedback(struct feedback *feedback)
 {
 	if (feedback->feedback)
-		presentation_feedback_destroy(feedback->feedback);
+		wp_presentation_feedback_destroy(feedback->feedback);
 
 	wl_list_remove(&feedback->link);
 	free(feedback);
@@ -341,7 +345,7 @@ paint_pixels(void *image, int width, int height, uint32_t phase)
 
 static void
 feedback_sync_output(void *data,
-		     struct presentation_feedback *presentation_feedback,
+		     struct wp_presentation_feedback *presentation_feedback,
 		     struct wl_output *output)
 {
 	/* not interested */
@@ -354,10 +358,10 @@ pflags_to_str(uint32_t flags, char *str, unsigned len)
 		uint32_t flag;
 		char sym;
 	} desc[] = {
-		{ PRESENTATION_FEEDBACK_KIND_VSYNC, 's' },
-		{ PRESENTATION_FEEDBACK_KIND_HW_CLOCK, 'c' },
-		{ PRESENTATION_FEEDBACK_KIND_HW_COMPLETION, 'e' },
-		{ PRESENTATION_FEEDBACK_KIND_ZERO_COPY, 'z' },
+		{ WP_PRESENTATION_FEEDBACK_KIND_VSYNC, 's' },
+		{ WP_PRESENTATION_FEEDBACK_KIND_HW_CLOCK, 'c' },
+		{ WP_PRESENTATION_FEEDBACK_KIND_HW_COMPLETION, 'e' },
+		{ WP_PRESENTATION_FEEDBACK_KIND_ZERO_COPY, 'z' },
 	};
 	unsigned i;
 
@@ -397,7 +401,7 @@ timespec_diff_to_usec(const struct timespec *a, const struct timespec *b)
 
 static void
 feedback_presented(void *data,
-		   struct presentation_feedback *presentation_feedback,
+		   struct wp_presentation_feedback *presentation_feedback,
 		   uint32_t tv_sec_hi,
 		   uint32_t tv_sec_lo,
 		   uint32_t tv_nsec,
@@ -453,7 +457,7 @@ feedback_presented(void *data,
 
 static void
 feedback_discarded(void *data,
-		   struct presentation_feedback *presentation_feedback)
+		   struct wp_presentation_feedback *presentation_feedback)
 {
 	struct feedback *feedback = data;
 
@@ -462,17 +466,34 @@ feedback_discarded(void *data,
 	destroy_feedback(feedback);
 }
 
-static const struct presentation_feedback_listener feedback_listener = {
+static const struct wp_presentation_feedback_listener feedback_listener = {
 	feedback_sync_output,
 	feedback_presented,
 	feedback_discarded
 };
 
+static void
+window_emulate_rendering(struct window *window)
+{
+	struct timespec delay;
+	int ret;
+
+	if (window->commit_delay_msecs <= 0)
+		return;
+
+	delay.tv_sec = window->commit_delay_msecs / 1000;
+	delay.tv_nsec = (window->commit_delay_msecs % 1000) * 1000000;
+
+	ret = nanosleep(&delay, NULL);
+	if (ret)
+		printf("nanosleep failed: %m\n");
+}
+
 static void
 window_create_feedback(struct window *window, uint32_t frame_stamp)
 {
 	static unsigned seq;
-	struct presentation *pres = window->display->presentation;
+	struct wp_presentation *pres = window->display->presentation;
 	struct feedback *feedback;
 
 	seq++;
@@ -480,16 +501,17 @@ window_create_feedback(struct window *window, uint32_t frame_stamp)
 	if (!pres)
 		return;
 
-	feedback = calloc(1, sizeof *feedback);
+	feedback = zalloc(sizeof *feedback);
 	if (!feedback)
 		return;
 
 	feedback->window = window;
-	feedback->feedback = presentation_feedback(pres, window->surface);
-	presentation_feedback_add_listener(feedback->feedback,
-					   &feedback_listener, feedback);
+	feedback->feedback = wp_presentation_feedback(pres, window->surface);
+	wp_presentation_feedback_add_listener(feedback->feedback,
+					      &feedback_listener, feedback);
 
 	feedback->frame_no = seq;
+
 	clock_gettime(window->display->clk_id, &feedback->commit);
 	feedback->frame_stamp = frame_stamp;
 	feedback->target = feedback->commit;
@@ -524,6 +546,8 @@ redraw_mode_feedback(void *data, struct wl_callback *callback, uint32_t time)
 	if (callback)
 		wl_callback_destroy(callback);
 
+	window_emulate_rendering(window);
+
 	window->callback = wl_surface_frame(window->surface);
 	wl_callback_add_listener(window->callback,
 				 &frame_listener_mode_feedback, window);
@@ -536,21 +560,22 @@ static const struct wl_callback_listener frame_listener_mode_feedback = {
 	redraw_mode_feedback
 };
 
-static const struct presentation_feedback_listener feedkick_listener;
+static const struct wp_presentation_feedback_listener feedkick_listener;
 
 static void
 window_feedkick(struct window *window)
 {
-	struct presentation *pres = window->display->presentation;
-	struct presentation_feedback *fback;
+	struct wp_presentation *pres = window->display->presentation;
+	struct wp_presentation_feedback *fback;
 
-	fback = presentation_feedback(pres, window->surface);
-	presentation_feedback_add_listener(fback, &feedkick_listener, window);
+	fback = wp_presentation_feedback(pres, window->surface);
+	wp_presentation_feedback_add_listener(fback, &feedkick_listener,
+					      window);
 }
 
 static void
 feedkick_presented(void *data,
-		   struct presentation_feedback *presentation_feedback,
+		   struct wp_presentation_feedback *presentation_feedback,
 		   uint32_t tv_sec_hi,
 		   uint32_t tv_sec_lo,
 		   uint32_t tv_nsec,
@@ -561,11 +586,12 @@ feedkick_presented(void *data,
 {
 	struct window *window = data;
 
-	presentation_feedback_destroy(presentation_feedback);
+	wp_presentation_feedback_destroy(presentation_feedback);
 	window->refresh_nsec = refresh_nsec;
 
 	switch (window->mode) {
 	case RUN_MODE_PRESENT:
+		window_emulate_rendering(window);
 		window_create_feedback(window, 0);
 		window_feedkick(window);
 		window_commit_next(window);
@@ -578,14 +604,15 @@ feedkick_presented(void *data,
 
 static void
 feedkick_discarded(void *data,
-		   struct presentation_feedback *presentation_feedback)
+		   struct wp_presentation_feedback *presentation_feedback)
 {
 	struct window *window = data;
 
-	presentation_feedback_destroy(presentation_feedback);
+	wp_presentation_feedback_destroy(presentation_feedback);
 
 	switch (window->mode) {
 	case RUN_MODE_PRESENT:
+		window_emulate_rendering(window);
 		window_create_feedback(window, 0);
 		window_feedkick(window);
 		window_commit_next(window);
@@ -596,7 +623,7 @@ feedkick_discarded(void *data,
 	}
 }
 
-static const struct presentation_feedback_listener feedkick_listener = {
+static const struct wp_presentation_feedback_listener feedkick_listener = {
 	feedback_sync_output,
 	feedkick_presented,
 	feedkick_discarded
@@ -605,6 +632,8 @@ static const struct presentation_feedback_listener feedkick_listener = {
 static void
 firstdraw_mode_burst(struct window *window)
 {
+	window_emulate_rendering(window);
+
 	switch (window->mode) {
 	case RUN_MODE_PRESENT:
 		window_create_feedback(window, 0);
@@ -650,7 +679,7 @@ display_add_output(struct display *d, uint32_t name, uint32_t version)
 {
 	struct output *o;
 
-	o = calloc(1, sizeof(*o));
+	o = zalloc(sizeof(*o));
 	assert(o);
 
 	o->output = wl_registry_bind(d->registry, name,
@@ -660,7 +689,7 @@ display_add_output(struct display *d, uint32_t name, uint32_t version)
 }
 
 static void
-presentation_clock_id(void *data, struct presentation *presentation,
+presentation_clock_id(void *data, struct wp_presentation *presentation,
 		      uint32_t clk_id)
 {
 	struct display *d = data;
@@ -668,7 +697,7 @@ presentation_clock_id(void *data, struct presentation *presentation,
 	d->clk_id = clk_id;
 }
 
-static const struct presentation_listener presentation_listener = {
+static const struct wp_presentation_listener presentation_listener = {
 	presentation_clock_id
 };
 
@@ -703,12 +732,12 @@ registry_handle_global(void *data, struct wl_registry *registry,
 		wl_shm_add_listener(d->shm, &shm_listener, d);
 	} else if (strcmp(interface, "wl_output") == 0) {
 		display_add_output(d, name, version);
-	} else if (strcmp(interface, "presentation") == 0) {
+	} else if (strcmp(interface, wp_presentation_interface.name) == 0) {
 		d->presentation =
 			wl_registry_bind(registry,
-					 name, &presentation_interface, 1);
-		presentation_add_listener(d->presentation,
-					  &presentation_listener, d);
+					 name, &wp_presentation_interface, 1);
+		wp_presentation_add_listener(d->presentation,
+					     &presentation_listener, d);
 	}
 }
 
@@ -807,10 +836,12 @@ usage(const char *prog, int exit_code)
 {
 	fprintf(stderr, "Usage: %s [mode] [options]\n"
 		"where 'mode' is one of\n"
-		"  -f\trun in feedback mode (default)\n"
-		"  -i\trun in feedback-idle mode; sleep 1s between frames\n"
-		"  -p\trun in low-latency presentation mode\n"
-		"and 'options' may include\n",
+		"  -f\t\trun in feedback mode (default)\n"
+		"  -i\t\trun in feedback-idle mode; sleep 1s between frames\n"
+		"  -p\t\trun in low-latency presentation mode\n"
+		"and 'options' may include\n"
+		"  -d msecs\temulate the time used for rendering by a delay \n"
+		"\t\tof the given milliseconds before commit\n\n",
 		prog);
 
 	fprintf(stderr, "Printed timing statistics, depending on mode:\n"
@@ -835,6 +866,7 @@ main(int argc, char **argv)
 	int ret = 0;
 	enum run_mode mode = RUN_MODE_FEEDBACK;
 	int i;
+	int commit_delay_msecs = 0;
 
 	for (i = 1; i < argc; i++) {
 		if (strcmp("-f", argv[i]) == 0)
@@ -843,12 +875,16 @@ main(int argc, char **argv)
 			mode = RUN_MODE_FEEDBACK_IDLE;
 		else if (strcmp("-p", argv[i]) == 0)
 			mode = RUN_MODE_PRESENT;
+		else if ((strcmp("-d", argv[i]) == 0) && (i + 1 < argc)) {
+			i++;
+			commit_delay_msecs = atoi(argv[i]);
+		}
 		else
 			usage(argv[0], EXIT_FAILURE);
 	}
 
 	display = create_display();
-	window = create_window(display, 250, 250, mode);
+	window = create_window(display, 250, 250, mode, commit_delay_msecs);
 	if (!window)
 		return 1;
 
