@@ -35,8 +35,9 @@
 
 #include <wayland-client.h>
 #include "shared/os-compatibility.h"
-#include "xdg-shell-client-protocol.h"
-#include "fullscreen-shell-client-protocol.h"
+#include "shared/zalloc.h"
+#include "xdg-shell-unstable-v5-client-protocol.h"
+#include "fullscreen-shell-unstable-v1-client-protocol.h"
 
 #include <sys/types.h>
 #include "ivi-application-client-protocol.h"
@@ -47,9 +48,9 @@ struct display {
 	struct wl_registry *registry;
 	struct wl_compositor *compositor;
 	struct xdg_shell *shell;
-	struct _wl_fullscreen_shell *fshell;
+	struct zwp_fullscreen_shell_v1 *fshell;
 	struct wl_shm *shm;
-	uint32_t formats;
+	bool has_xrgb;
 	struct ivi_application *ivi_application;
 };
 
@@ -157,7 +158,7 @@ create_window(struct display *display, int width, int height)
 {
 	struct window *window;
 
-	window = calloc(1, sizeof *window);
+	window = zalloc(sizeof *window);
 	if (!window)
 		return NULL;
 
@@ -180,10 +181,10 @@ create_window(struct display *display, int width, int height)
 		xdg_surface_set_title(window->xdg_surface, "simple-shm");
 
 	} else if (display->fshell) {
-		_wl_fullscreen_shell_present_surface(display->fshell,
-						     window->surface,
-						     _WL_FULLSCREEN_SHELL_PRESENT_METHOD_DEFAULT,
-						     NULL);
+		zwp_fullscreen_shell_v1_present_surface(display->fshell,
+							window->surface,
+							ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_DEFAULT,
+							NULL);
 	} else if (display->ivi_application ) {
 		uint32_t id_ivisurf = IVI_SURFACE_ID + (uint32_t)getpid();
 		window->ivi_surface =
@@ -336,7 +337,8 @@ shm_format(void *data, struct wl_shm *wl_shm, uint32_t format)
 {
 	struct display *d = data;
 
-	d->formats |= (1 << format);
+	if (format == WL_SHM_FORMAT_XRGB8888)
+		d->has_xrgb = true;
 }
 
 struct wl_shm_listener shm_listener = {
@@ -374,9 +376,9 @@ registry_handle_global(void *data, struct wl_registry *registry,
 					    id, &xdg_shell_interface, 1);
 		xdg_shell_use_unstable_version(d->shell, XDG_VERSION);
 		xdg_shell_add_listener(d->shell, &xdg_shell_listener, d);
-	} else if (strcmp(interface, "_wl_fullscreen_shell") == 0) {
+	} else if (strcmp(interface, "zwp_fullscreen_shell_v1") == 0) {
 		d->fshell = wl_registry_bind(registry,
-					     id, &_wl_fullscreen_shell_interface, 1);
+					     id, &zwp_fullscreen_shell_v1_interface, 1);
 	} else if (strcmp(interface, "wl_shm") == 0) {
 		d->shm = wl_registry_bind(registry,
 					  id, &wl_shm_interface, 1);
@@ -413,7 +415,7 @@ create_display(void)
 	display->display = wl_display_connect(NULL);
 	assert(display->display);
 
-	display->formats = 0;
+	display->has_xrgb = false;
 	display->registry = wl_display_get_registry(display->display);
 	wl_registry_add_listener(display->registry,
 				 &registry_listener, display);
@@ -465,7 +467,7 @@ create_display(void)
 	 * technique.
 	 */
 
-	if (!(display->formats & (1 << WL_SHM_FORMAT_XRGB8888))) {
+	if (!display->has_xrgb) {
 		fprintf(stderr, "WL_SHM_FORMAT_XRGB32 not available\n");
 		exit(1);
 	}
@@ -483,7 +485,7 @@ destroy_display(struct display *display)
 		xdg_shell_destroy(display->shell);
 
 	if (display->fshell)
-		_wl_fullscreen_shell_release(display->fshell);
+		zwp_fullscreen_shell_v1_release(display->fshell);
 
 	if (display->compositor)
 		wl_compositor_destroy(display->compositor);
