@@ -23,11 +23,15 @@
  * SOFTWARE.
  */
 
+#include "config.h"
+
 #include <time.h>
 #include <assert.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <stdbool.h>
 
+#include "ivi-shell.h"
 #include "ivi-layout-export.h"
 #include "ivi-layout-private.h"
 
@@ -98,6 +102,22 @@ is_surface_transition(struct ivi_layout_surface *surface)
 	return 0;
 }
 
+void
+ivi_layout_remove_all_surface_transitions(struct ivi_layout_surface *surface)
+{
+	struct ivi_layout *layout = get_instance();
+	struct transition_node *node;
+	struct transition_node *tmp;
+	struct ivi_layout_transition *tran;
+
+	wl_list_for_each_safe(node, tmp, &layout->transitions->transition_list, link) {
+		tran = node->transition;
+		if (tran->is_transition_func(tran->private_data, surface)) {
+			layout_transition_destroy(tran);
+		}
+	};
+}
+
 static void
 tick_transition(struct ivi_layout_transition *transition, uint32_t timestamp)
 {
@@ -181,7 +201,7 @@ ivi_layout_transition_set_create(struct weston_compositor *ec)
 	return transitions;
 }
 
-static void
+static bool
 layout_transition_register(struct ivi_layout_transition *trans)
 {
 	struct ivi_layout *layout = get_instance();
@@ -190,11 +210,12 @@ layout_transition_register(struct ivi_layout_transition *trans)
 	node = malloc(sizeof(*node));
 	if (node == NULL) {
 		weston_log("%s: memory allocation fails\n", __func__);
-		return;
+		return false;
 	}
 
 	node->transition = trans;
 	wl_list_insert(&layout->pending_transition_list, &node->link);
+	return true;
 }
 
 static void
@@ -251,6 +272,7 @@ create_layout_transition(void)
 
 	transition->is_done = 0;
 
+	transition->is_transition_func = NULL;
 	transition->private_data = NULL;
 	transition->user_data = NULL;
 
@@ -281,7 +303,9 @@ transition_move_resize_view_destroy(struct ivi_layout_transition *transition)
 		(struct move_resize_view_data *)transition->private_data;
 	struct ivi_layout_surface *layout_surface = data->surface;
 
-	wl_signal_emit(&layout_surface->configured, layout_surface);
+	shell_surface_send_configure(layout_surface->surface,
+				     layout_surface->prop.dest_width,
+				     layout_surface->prop.dest_height);
 
 	if (transition->private_data) {
 		free(transition->private_data);
@@ -340,6 +364,7 @@ create_move_resize_view_transition(
 	data = malloc(sizeof(*data));
 	if (data == NULL) {
 		weston_log("%s: memory allocation fails\n", __func__);
+		free(transition);
 		return NULL;
 	}
 
@@ -414,7 +439,9 @@ ivi_layout_transition_move_resize_view(struct ivi_layout_surface *surface,
 		transition_move_resize_view_destroy,
 		duration);
 
-	layout_transition_register(transition);
+	if(transition && layout_transition_register(transition))
+		return;
+	layout_transition_destroy(transition);
 }
 
 /* fade transition */
@@ -468,6 +495,7 @@ create_fade_view_transition(
 	data = malloc(sizeof(*data));
 	if (data == NULL) {
 		weston_log("%s: memory allocation fails\n", __func__);
+		free(transition);
 		return NULL;
 	}
 
@@ -507,7 +535,9 @@ create_visibility_transition(struct ivi_layout_surface *surface,
 		destroy_func,
 		duration);
 
-	layout_transition_register(transition);
+	if (transition && layout_transition_register(transition))
+		return;
+	layout_transition_destroy(transition);
 }
 
 static void
@@ -530,8 +560,8 @@ ivi_layout_transition_visibility_on(struct ivi_layout_surface *surface,
 				    uint32_t duration)
 {
 	struct ivi_layout_transition *transition;
-	bool is_visible = ivi_layout_surface_get_visibility(surface);
-	wl_fixed_t dest_alpha = ivi_layout_surface_get_opacity(surface);
+	bool is_visible = surface->prop.visibility;
+	wl_fixed_t dest_alpha = surface->prop.opacity;
 	struct store_alpha *user_data = NULL;
 	wl_fixed_t start_alpha = 0.0;
 	struct fade_view_data *data = NULL;
@@ -540,7 +570,7 @@ ivi_layout_transition_visibility_on(struct ivi_layout_surface *surface,
 					IVI_LAYOUT_TRANSITION_VIEW_FADE,
 					surface);
 	if (transition) {
-		start_alpha = ivi_layout_surface_get_opacity(surface);
+		start_alpha = surface->prop.opacity;
 		user_data = transition->user_data;
 		data = transition->private_data;
 
@@ -595,7 +625,7 @@ ivi_layout_transition_visibility_off(struct ivi_layout_surface *surface,
 				     uint32_t duration)
 {
 	struct ivi_layout_transition *transition;
-	wl_fixed_t start_alpha = ivi_layout_surface_get_opacity(surface);
+	wl_fixed_t start_alpha = surface->prop.opacity;
 	struct store_alpha* user_data = NULL;
 	struct fade_view_data* data = NULL;
 
@@ -655,7 +685,8 @@ transition_move_layer_user_frame(struct ivi_layout_transition *transition)
 	const int32_t dest_y = data->start_y +
 		(data->end_y - data->start_y) * current;
 
-	ivi_layout_layer_set_position(layer, dest_x, dest_y);
+	ivi_layout_layer_set_destination_rectangle(layer, dest_x, dest_y,
+			layer->prop.dest_width, layer->prop.dest_height);
 }
 
 static void
@@ -697,6 +728,7 @@ create_move_layer_transition(
 	data = malloc(sizeof(*data));
 	if (data == NULL) {
 		weston_log("%s: memory allocation fails\n", __func__);
+		free(transition);
 		return NULL;
 	}
 
@@ -726,12 +758,10 @@ ivi_layout_transition_move_layer(struct ivi_layout_layer *layer,
 				 int32_t dest_x, int32_t dest_y,
 				 uint32_t duration)
 {
-	int32_t start_pos_x = 0;
-	int32_t start_pos_y = 0;
+	int32_t start_pos_x = layer->prop.dest_x;
+	int32_t start_pos_y = layer->prop.dest_y;
 	struct ivi_layout_transition *transition = NULL;
 
-	ivi_layout_layer_get_position(layer, &start_pos_x, &start_pos_y);
-
 	transition = create_move_layer_transition(
 		layer,
 		start_pos_x, start_pos_y,
@@ -739,9 +769,10 @@ ivi_layout_transition_move_layer(struct ivi_layout_layer *layer,
 		NULL, NULL,
 		duration);
 
-	layout_transition_register(transition);
+	if (transition && layout_transition_register(transition))
+		return;
 
-	return;
+	free(transition);
 }
 
 void
@@ -820,7 +851,7 @@ ivi_layout_transition_fade_layer(
 		data = transition->private_data;
 
 		/* FIXME */
-		fixed_opacity = ivi_layout_layer_get_opacity(layer);
+		fixed_opacity = layer->prop.opacity;
 		now_opacity = wl_fixed_to_double(fixed_opacity);
 		remain = 0.0;
 
@@ -843,6 +874,7 @@ ivi_layout_transition_fade_layer(
 	data = malloc(sizeof(*data));
 	if (data == NULL) {
 		weston_log("%s: memory allocation fails\n", __func__);
+		free(transition);
 		return;
 	}
 
@@ -864,7 +896,8 @@ ivi_layout_transition_fade_layer(
 	data->end_alpha = end_alpha;
 	data->destroy_func = destroy_func;
 
-	layout_transition_register(transition);
+	if (!layout_transition_register(transition))
+		layout_transition_destroy(transition);
 
 	return;
 }
