@@ -44,7 +44,7 @@
 #include "ivi-shell.h"
 #include "ivi-application-server-protocol.h"
 #include "ivi-layout-export.h"
-#include "ivi-layout-private.h"
+#include "ivi-layout-shell.h"
 #include "shared/helpers.h"
 
 /* Representation of ivi_surface protocol object. */
@@ -63,8 +63,6 @@ struct ivi_shell_surface
 	int32_t height;
 
 	struct wl_list link;
-
-	struct wl_listener configured_listener;
 };
 
 struct ivi_shell_setting
@@ -77,38 +75,37 @@ struct ivi_shell_setting
  * Implementation of ivi_surface
  */
 
-static void
-surface_configure_notify(struct wl_listener *listener, void *data)
-{
-	struct ivi_layout_surface *layout_surf =
-		(struct ivi_layout_surface *)data;
-
-	struct ivi_shell_surface *shell_surf =
-		container_of(listener,
-			     struct ivi_shell_surface,
-			     configured_listener);
-
-	int32_t dest_width = 0;
-	int32_t dest_height = 0;
-
-	ivi_layout_surface_get_dimension(layout_surf,
-					 &dest_width, &dest_height);
-
-	if (shell_surf->resource)
-		ivi_surface_send_configure(shell_surf->resource,
-					   dest_width, dest_height);
-}
-
 static void
 ivi_shell_surface_configure(struct weston_surface *, int32_t, int32_t);
 
 static struct ivi_shell_surface *
 get_ivi_shell_surface(struct weston_surface *surface)
 {
-	if (surface->configure == ivi_shell_surface_configure)
-		return surface->configure_private;
+	struct ivi_shell_surface *shsurf;
 
-	return NULL;
+	if (surface->configure != ivi_shell_surface_configure)
+		return NULL;
+
+	shsurf = surface->configure_private;
+	assert(shsurf);
+	assert(shsurf->surface == surface);
+
+	return shsurf;
+}
+
+void
+shell_surface_send_configure(struct weston_surface *surface,
+			     int32_t width, int32_t height)
+{
+	struct ivi_shell_surface *shsurf;
+
+	shsurf = get_ivi_shell_surface(surface);
+	assert(shsurf);
+	if (!shsurf)
+		return;
+
+	if (shsurf->resource)
+		ivi_surface_send_configure(shsurf->resource, width, height);
 }
 
 static void
@@ -117,7 +114,11 @@ ivi_shell_surface_configure(struct weston_surface *surface,
 {
 	struct ivi_shell_surface *ivisurf = get_ivi_shell_surface(surface);
 
-	if (surface->width == 0 || surface->height == 0 || ivisurf == NULL)
+	assert(ivisurf);
+	if (!ivisurf)
+		return;
+
+	if (surface->width == 0 || surface->height == 0)
 		return;
 
 	if (ivisurf->width != surface->width ||
@@ -130,6 +131,19 @@ ivi_shell_surface_configure(struct weston_surface *surface,
 	}
 }
 
+static int
+ivi_shell_surface_get_label(struct weston_surface *surface,
+			    char *buf,
+			    size_t len)
+{
+	struct ivi_shell_surface *shell_surf = get_ivi_shell_surface(surface);
+
+	if (!shell_surf)
+		return snprintf(buf, len, "unidentified window in ivi-shell");
+
+	return snprintf(buf, len, "ivi-surface %#x", shell_surf->id_surface);
+}
+
 static void
 layout_surface_cleanup(struct ivi_shell_surface *ivisurf)
 {
@@ -140,6 +154,7 @@ layout_surface_cleanup(struct ivi_shell_surface *ivisurf)
 
 	ivisurf->surface->configure = NULL;
 	ivisurf->surface->configure_private = NULL;
+	weston_surface_set_label_func(ivisurf->surface, NULL);
 	ivisurf->surface = NULL;
 
 	// destroy weston_surface destroy signal.
@@ -258,9 +273,7 @@ application_surface_create(struct wl_client *client,
 	ivisurf->width = 0;
 	ivisurf->height = 0;
 	ivisurf->layout_surface = layout_surface;
-	ivisurf->configured_listener.notify = surface_configure_notify;
-	ivi_layout_surface_add_configured_listener(layout_surface,
-				     &ivisurf->configured_listener);
+
 	/*
 	 * The following code relies on wl_surface destruction triggering
 	 * immediateweston_surface destruction
@@ -273,6 +286,8 @@ application_surface_create(struct wl_client *client,
 
 	weston_surface->configure = ivi_shell_surface_configure;
 	weston_surface->configure_private = ivisurf;
+	weston_surface_set_label_func(weston_surface,
+				      ivi_shell_surface_get_label);
 
 	res = wl_resource_create(client, &ivi_surface_interface, 1, id);
 	if (res == NULL) {
@@ -414,6 +429,59 @@ ivi_shell_setting_create(struct ivi_shell_setting *dest,
 	return result;
 }
 
+static void
+activate_binding(struct weston_seat *seat,
+		 struct weston_view *focus_view)
+{
+	struct weston_surface *focus = focus_view->surface;
+	struct weston_surface *main_surface =
+		weston_surface_get_main_surface(focus);
+
+	if (get_ivi_shell_surface(main_surface) == NULL)
+		return;
+
+	weston_surface_activate(focus, seat);
+}
+
+static void
+click_to_activate_binding(struct weston_pointer *pointer, uint32_t time,
+			  uint32_t button, void *data)
+{
+	if (pointer->grab != &pointer->default_grab)
+		return;
+	if (pointer->focus == NULL)
+		return;
+
+	activate_binding(pointer->seat, pointer->focus);
+}
+
+static void
+touch_to_activate_binding(struct weston_touch *touch, uint32_t time,
+			  void *data)
+{
+	if (touch->grab != &touch->default_grab)
+		return;
+	if (touch->focus == NULL)
+		return;
+
+	activate_binding(touch->seat, touch->focus);
+}
+
+static void
+shell_add_bindings(struct weston_compositor *compositor,
+		   struct ivi_shell *shell)
+{
+	weston_compositor_add_button_binding(compositor, BTN_LEFT, 0,
+					     click_to_activate_binding,
+					     shell);
+	weston_compositor_add_button_binding(compositor, BTN_RIGHT, 0,
+					     click_to_activate_binding,
+					     shell);
+	weston_compositor_add_touch_binding(compositor, 0,
+					    touch_to_activate_binding,
+					    shell);
+}
+
 /*
  * Initialization of ivi-shell.
  */
@@ -450,6 +518,7 @@ module_init(struct weston_compositor *compositor,
 		goto out_settings;
 
 	ivi_layout_init_with_compositor(compositor);
+	shell_add_bindings(compositor, shell);
 
 	/* Call module_init of ivi-modules which are defined in weston.ini */
 	if (load_controller_modules(compositor, setting.ivi_module,
