@@ -30,13 +30,16 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <jpeglib.h>
 #include <png.h>
 #include <pixman.h>
 
 #include "shared/helpers.h"
 #include "image-loader.h"
 
+#ifdef HAVE_JPEG
+#include <jpeglib.h>
+#endif
+
 #ifdef HAVE_WEBP
 #include <webp/decode.h>
 #endif
@@ -47,6 +50,14 @@ stride_for_width(int width)
 	return width * 4;
 }
 
+static void
+pixman_image_destroy_func(pixman_image_t *image, void *data)
+{
+	free(data);
+}
+
+#ifdef HAVE_JPEG
+
 static void
 swizzle_row(JSAMPLE *row, JDIMENSION width)
 {
@@ -68,12 +79,6 @@ error_exit(j_common_ptr cinfo)
 	longjmp(cinfo->client_data, 1);
 }
 
-static void
-pixman_image_destroy_func(pixman_image_t *image, void *data)
-{
-	free(data);
-}
-
 static pixman_image_t *
 load_jpeg(FILE *fp)
 {
@@ -132,6 +137,17 @@ load_jpeg(FILE *fp)
 	return pixman_image;
 }
 
+#else
+
+static pixman_image_t *
+load_jpeg(FILE *fp)
+{
+	fprintf(stderr, "JPEG support disabled at compile-time\n");
+	return NULL;
+}
+
+#endif
+
 static inline int
 multiply_alpha(int alpha, int color)
 {
@@ -352,6 +368,15 @@ load_webp(FILE *fp)
 					config.output.u.RGBA.stride);
 }
 
+#else
+
+static pixman_image_t *
+load_webp(FILE *fp)
+{
+	fprintf(stderr, "WebP support disabled at compile-time\n");
+	return NULL;
+}
+
 #endif
 
 
@@ -364,9 +389,7 @@ struct image_loader {
 static const struct image_loader loaders[] = {
 	{ { 0x89, 'P', 'N', 'G' }, 4, load_png },
 	{ { 0xff, 0xd8 }, 2, load_jpeg },
-#ifdef HAVE_WEBP
 	{ { 'R', 'I', 'F', 'F' }, 4, load_webp }
-#endif
 };
 
 pixman_image_t *
