@@ -46,16 +46,17 @@
 #include <gbm.h>
 #include <libudev.h>
 
+#include "compositor.h"
+#include "compositor-drm.h"
 #include "shared/helpers.h"
 #include "shared/timespec-util.h"
-#include "libbacklight.h"
-#include "compositor.h"
 #include "gl-renderer.h"
 #include "pixman-renderer.h"
+#include "libbacklight.h"
 #include "libinput-seat.h"
 #include "launcher-util.h"
 #include "vaapi-recorder.h"
-#include "presentation_timing-server-protocol.h"
+#include "presentation-time-server-protocol.h"
 #include "linux-dmabuf.h"
 
 #ifndef DRM_CAP_TIMESTAMP_MONOTONIC
@@ -74,17 +75,6 @@
 #define GBM_BO_USE_CURSOR GBM_BO_USE_CURSOR_64X64
 #endif
 
-static int option_current_mode = 0;
-
-enum output_config {
-	OUTPUT_CONFIG_INVALID = 0,
-	OUTPUT_CONFIG_OFF,
-	OUTPUT_CONFIG_PREFERRED,
-	OUTPUT_CONFIG_CURRENT,
-	OUTPUT_CONFIG_MODE,
-	OUTPUT_CONFIG_MODELINE
-};
-
 struct drm_backend {
 	struct weston_backend base;
 	struct weston_compositor *compositor;
@@ -106,7 +96,7 @@ struct drm_backend {
 	uint32_t crtc_allocator;
 	uint32_t connector_allocator;
 	struct wl_listener session_listener;
-	uint32_t format;
+	uint32_t gbm_format;
 
 	/* we need these parameters in order to not fail drmModeAddFB2()
 	 * due to out of bounds dimensions, and then mistakenly set
@@ -130,6 +120,18 @@ struct drm_backend {
 
 	int32_t cursor_width;
 	int32_t cursor_height;
+
+        /** Callback used to configure the outputs.
+	 *
+         * This function will be called by the backend when a new DRM
+         * output needs to be configured.
+         */
+        enum weston_drm_backend_output_mode
+	(*configure_output)(struct weston_compositor *compositor,
+			    bool use_current_mode,
+			    const char *name,
+			    struct weston_drm_backend_output_config *output_config);
+	bool use_current_mode;
 };
 
 struct drm_mode {
@@ -137,10 +139,7 @@ struct drm_mode {
 	drmModeModeInfo mode_info;
 };
 
-struct drm_output;
-
 struct drm_fb {
-	struct drm_output *output;
 	uint32_t fb_id, stride, handle, size;
 	int fd;
 	int is_client_buffer;
@@ -169,7 +168,7 @@ struct drm_output {
 	drmModeCrtcPtr original_crtc;
 	struct drm_edid edid;
 	drmModePropertyPtr dpms_prop;
-	uint32_t format;
+	uint32_t gbm_format;
 
 	enum dpms_enum dpms;
 
@@ -177,8 +176,8 @@ struct drm_output {
 	int page_flip_pending;
 	int destroy_pending;
 
-	struct gbm_surface *surface;
-	struct gbm_bo *cursor_bo[2];
+	struct gbm_surface *gbm_surface;
+	struct gbm_bo *gbm_cursor_bo[2];
 	struct weston_plane cursor_plane;
 	struct weston_plane fb_plane;
 	struct weston_view *cursor_view;
@@ -220,13 +219,6 @@ struct drm_sprite {
 	uint32_t formats[];
 };
 
-struct drm_parameters {
-	int connector;
-	int tty;
-	int use_pixman;
-	const char *seat_id;
-};
-
 static struct gl_renderer_interface *gl_renderer;
 
 static const char default_seat[] = "seat0";
@@ -308,7 +300,7 @@ drm_fb_create_dumb(struct drm_backend *b, unsigned width, unsigned height)
 	if (ret)
 		goto err_add_fb;
 
-	fb->map = mmap(0, fb->size, PROT_WRITE,
+	fb->map = mmap(NULL, fb->size, PROT_WRITE,
 		       MAP_SHARED, b->drm.fd, map_arg.offset);
 	if (fb->map == MAP_FAILED)
 		goto err_add_fb;
@@ -438,7 +430,7 @@ drm_output_release_fb(struct drm_output *output, struct drm_fb *fb)
 		if (fb->is_client_buffer)
 			gbm_bo_destroy(fb->bo);
 		else
-			gbm_surface_release_buffer(output->surface,
+			gbm_surface_release_buffer(output->gbm_surface,
 						   fb->bo);
 	}
 }
@@ -467,7 +459,7 @@ drm_output_check_scanout_format(struct drm_output *output,
 		pixman_region32_fini(&r);
 	}
 
-	if (output->format == format)
+	if (output->gbm_format == format)
 		return format;
 
 	return 0;
@@ -530,16 +522,16 @@ drm_output_render_gl(struct drm_output *output, pixman_region32_t *damage)
 	output->base.compositor->renderer->repaint_output(&output->base,
 							  damage);
 
-	bo = gbm_surface_lock_front_buffer(output->surface);
+	bo = gbm_surface_lock_front_buffer(output->gbm_surface);
 	if (!bo) {
 		weston_log("failed to lock front buffer: %m\n");
 		return;
 	}
 
-	output->next = drm_fb_get_from_bo(bo, b, output->format);
+	output->next = drm_fb_get_from_bo(bo, b, output->gbm_format);
 	if (!output->next) {
 		weston_log("failed to get drm_fb for bo\n");
-		gbm_surface_release_buffer(output->surface, bo);
+		gbm_surface_release_buffer(output->gbm_surface, bo);
 		return;
 	}
 }
@@ -608,13 +600,13 @@ drm_output_set_gamma(struct weston_output *output_base,
 }
 
 /* Determine the type of vblank synchronization to use for the output.
- * 
+ *
  * The pipe parameter indicates which CRTC is in use.  Knowing this, we
  * can determine which vblank sequence type to use for it.  Traditional
  * cards had only two CRTCs, with CRTC 0 using no special flags, and
  * CRTC 1 using DRM_VBLANK_SECONDARY.  The first bit of the pipe
  * parameter indicates this.
- * 
+ *
  * Bits 1-5 of the pipe parameter are 5 bit wide pipe number between
  * 0-31.  If this is non-zero it indicates we're dealing with a
  * multi-gpu situation and we need to calculate the vblank sync
@@ -778,7 +770,7 @@ drm_output_start_repaint_loop(struct weston_output *output_base)
 		if (timespec_to_nsec(&vbl2now) < refresh_nsec) {
 			drm_output_update_msc(output, vbl.reply.sequence);
 			weston_output_finish_frame(output_base, &ts,
-						PRESENTATION_FEEDBACK_INVALID);
+						WP_PRESENTATION_FEEDBACK_INVALID);
 			return;
 		}
 	}
@@ -800,7 +792,7 @@ finish_frame:
 	/* if we cannot page-flip, immediately finish frame */
 	weston_compositor_read_presentation_clock(output_base->compositor, &ts);
 	weston_output_finish_frame(output_base, &ts,
-				   PRESENTATION_FEEDBACK_INVALID);
+				   WP_PRESENTATION_FEEDBACK_INVALID);
 }
 
 static void
@@ -821,8 +813,8 @@ vblank_handler(int fd, unsigned int frame, unsigned int sec, unsigned int usec,
 	struct drm_sprite *s = (struct drm_sprite *)data;
 	struct drm_output *output = s->output;
 	struct timespec ts;
-	uint32_t flags = PRESENTATION_FEEDBACK_KIND_HW_COMPLETION |
-			 PRESENTATION_FEEDBACK_KIND_HW_CLOCK;
+	uint32_t flags = WP_PRESENTATION_FEEDBACK_KIND_HW_COMPLETION |
+			 WP_PRESENTATION_FEEDBACK_KIND_HW_CLOCK;
 
 	drm_output_update_msc(output, frame);
 	output->vblank_pending = 0;
@@ -847,9 +839,9 @@ page_flip_handler(int fd, unsigned int frame,
 {
 	struct drm_output *output = (struct drm_output *) data;
 	struct timespec ts;
-	uint32_t flags = PRESENTATION_FEEDBACK_KIND_VSYNC |
-			 PRESENTATION_FEEDBACK_KIND_HW_COMPLETION |
-			 PRESENTATION_FEEDBACK_KIND_HW_CLOCK;
+	uint32_t flags = WP_PRESENTATION_FEEDBACK_KIND_VSYNC |
+			 WP_PRESENTATION_FEEDBACK_KIND_HW_COMPLETION |
+			 WP_PRESENTATION_FEEDBACK_KIND_HW_CLOCK;
 
 	drm_output_update_msc(output, frame);
 
@@ -983,14 +975,14 @@ drm_output_prepare_overlay_view(struct drm_output *output,
 		 * support a mix of gbm_bos and drmfbs.
 		 */
 		struct gbm_import_fd_data gbm_dmabuf = {
-			.fd     = dmabuf->dmabuf_fd[0],
-			.width  = dmabuf->width,
-			.height = dmabuf->height,
-			.stride = dmabuf->stride[0],
-			.format = dmabuf->format
+			.fd     = dmabuf->attributes.fd[0],
+			.width  = dmabuf->attributes.width,
+			.height = dmabuf->attributes.height,
+			.stride = dmabuf->attributes.stride[0],
+			.format = dmabuf->attributes.format
 		};
 
-		if (dmabuf->n_planes != 1 || dmabuf->offset[0] != 0)
+		if (dmabuf->attributes.n_planes != 1 || dmabuf->attributes.offset[0] != 0)
 			return NULL;
 
 		bo = gbm_bo_import(b->gbm, GBM_BO_IMPORT_FD, &gbm_dmabuf,
@@ -1094,7 +1086,11 @@ drm_output_prepare_cursor_view(struct drm_output *output,
 	struct drm_backend *b =
 		(struct drm_backend *)output->base.compositor->backend;
 	struct weston_buffer_viewport *viewport = &ev->surface->buffer_viewport;
+	struct wl_shm_buffer *shmbuf;
 
+	if (ev->transform.enabled &&
+	    (ev->transform.matrix.type > WESTON_MATRIX_TRANSFORM_TRANSLATE))
+		return NULL;
 	if (b->gbm == NULL)
 		return NULL;
 	if (output->base.transform != WL_OUTPUT_TRANSFORM_NORMAL)
@@ -1109,9 +1105,14 @@ drm_output_prepare_cursor_view(struct drm_output *output,
 		return NULL;
 	if (ev->geometry.scissor_enabled)
 		return NULL;
-	if (ev->surface->buffer_ref.buffer == NULL ||
-	    !wl_shm_buffer_get(ev->surface->buffer_ref.buffer->resource) ||
-	    ev->surface->width > b->cursor_width ||
+	if (ev->surface->buffer_ref.buffer == NULL)
+		return NULL;
+	shmbuf = wl_shm_buffer_get(ev->surface->buffer_ref.buffer->resource);
+	if (!shmbuf)
+		return NULL;
+	if (wl_shm_buffer_get_format(shmbuf) != WL_SHM_FORMAT_ARGB8888)
+		return NULL;
+	if (ev->surface->width > b->cursor_width ||
 	    ev->surface->height > b->cursor_height)
 		return NULL;
 
@@ -1166,11 +1167,13 @@ drm_output_set_cursor(struct drm_output *output)
 		(struct drm_backend *) output->base.compositor->backend;
 	EGLint handle;
 	struct gbm_bo *bo;
-	int x, y;
+	float x, y;
 
 	output->cursor_view = NULL;
 	if (ev == NULL) {
 		drmModeSetCursor(b->drm.fd, output->crtc_id, 0, 0, 0);
+		output->cursor_plane.x = INT32_MIN;
+		output->cursor_plane.y = INT32_MIN;
 		return;
 	}
 
@@ -1181,7 +1184,7 @@ drm_output_set_cursor(struct drm_output *output)
 		pixman_region32_fini(&output->cursor_plane.damage);
 		pixman_region32_init(&output->cursor_plane.damage);
 		output->current_cursor ^= 1;
-		bo = output->cursor_bo[output->current_cursor];
+		bo = output->gbm_cursor_bo[output->current_cursor];
 
 		cursor_bo_update(b, bo, ev);
 		handle = gbm_bo_get_handle(bo).s32;
@@ -1192,8 +1195,14 @@ drm_output_set_cursor(struct drm_output *output)
 		}
 	}
 
-	x = (ev->geometry.x - output->base.x) * output->base.current_scale;
-	y = (ev->geometry.y - output->base.y) * output->base.current_scale;
+	weston_view_to_global_float(ev, 0, 0, &x, &y);
+
+	/* From global to output space, output transform is guaranteed to be
+	 * NORMAL by drm_output_prepare_cursor_view().
+	 */
+	x = (x - output->base.x) * output->base.current_scale;
+	y = (y - output->base.y) * output->base.current_scale;
+
 	if (output->cursor_plane.x != x || output->cursor_plane.y != y) {
 		if (drmModeMoveCursor(b->drm.fd, output->crtc_id, x, y)) {
 			weston_log("failed to move cursor: %m\n");
@@ -1245,7 +1254,8 @@ drm_assign_planes(struct weston_output *output_base)
 		if (b->use_pixman ||
 		    (es->buffer_ref.buffer &&
 		    (!wl_shm_buffer_get(es->buffer_ref.buffer->resource) ||
-		     (ev->surface->width <= 64 && ev->surface->height <= 64))))
+		     (ev->surface->width <= b->cursor_width &&
+		      ev->surface->height <= b->cursor_height))))
 			es->keep_buffer = true;
 		else
 			es->keep_buffer = false;
@@ -1280,7 +1290,7 @@ drm_assign_planes(struct weston_output *output_base)
 			/* All other planes are a direct scanout of a
 			 * single client buffer.
 			 */
-			ev->psf_flags = PRESENTATION_FEEDBACK_KIND_ZERO_COPY;
+			ev->psf_flags = WP_PRESENTATION_FEEDBACK_KIND_ZERO_COPY;
 		}
 
 		pixman_region32_fini(&surface_overlap);
@@ -1326,7 +1336,7 @@ drm_output_destroy(struct weston_output *output_base)
 		drm_output_fini_pixman(output);
 	} else {
 		gl_renderer->output_destroy(output_base);
-		gbm_surface_destroy(output->surface);
+		gbm_surface_destroy(output->gbm_surface);
 	}
 
 	weston_plane_release(&output->fb_plane);
@@ -1427,7 +1437,7 @@ drm_output_switch_mode(struct weston_output *output_base, struct weston_mode *mo
 		}
 	} else {
 		gl_renderer->output_destroy(&output->base);
-		gbm_surface_destroy(output->surface);
+		gbm_surface_destroy(output->gbm_surface);
 
 		if (drm_output_init_egl(output, b) < 0) {
 			weston_log("failed to init output egl state with "
@@ -1533,7 +1543,7 @@ create_gbm_device(int fd)
 }
 
 /* When initializing EGL, if the preferred buffer format isn't available
- * we may be able to susbstitute an ARGB format for an XRGB one.
+ * we may be able to substitute an ARGB format for an XRGB one.
  *
  * This returns 0 if substitution isn't possible, but 0 might be a
  * legitimate format for other EGL platforms, so the caller is
@@ -1558,14 +1568,15 @@ fallback_format_for(uint32_t format)
 static int
 drm_backend_create_gl_renderer(struct drm_backend *b)
 {
-	EGLint format[2] = {
-		b->format,
-		fallback_format_for(b->format),
+	EGLint format[3] = {
+		b->gbm_format,
+		fallback_format_for(b->gbm_format),
+		0,
 	};
-	int n_formats = 1;
+	int n_formats = 2;
 
 	if (format[1])
-		n_formats = 2;
+		n_formats = 3;
 	if (gl_renderer->create(b->compositor,
 				EGL_PLATFORM_GBM_KHR,
 				(void *)b->gbm,
@@ -1815,18 +1826,18 @@ static int
 drm_output_init_egl(struct drm_output *output, struct drm_backend *b)
 {
 	EGLint format[2] = {
-		output->format,
-		fallback_format_for(output->format),
+		output->gbm_format,
+		fallback_format_for(output->gbm_format),
 	};
 	int i, flags, n_formats = 1;
 
-	output->surface = gbm_surface_create(b->gbm,
+	output->gbm_surface = gbm_surface_create(b->gbm,
 					     output->base.current_mode->width,
 					     output->base.current_mode->height,
 					     format[0],
 					     GBM_BO_USE_SCANOUT |
 					     GBM_BO_USE_RENDERING);
-	if (!output->surface) {
+	if (!output->gbm_surface) {
 		weston_log("failed to create gbm surface\n");
 		return -1;
 	}
@@ -1834,28 +1845,28 @@ drm_output_init_egl(struct drm_output *output, struct drm_backend *b)
 	if (format[1])
 		n_formats = 2;
 	if (gl_renderer->output_create(&output->base,
-				       (EGLNativeWindowType)output->surface,
-				       output->surface,
+				       (EGLNativeWindowType)output->gbm_surface,
+				       output->gbm_surface,
 				       gl_renderer->opaque_attribs,
 				       format,
 				       n_formats) < 0) {
 		weston_log("failed to create gl renderer output state\n");
-		gbm_surface_destroy(output->surface);
+		gbm_surface_destroy(output->gbm_surface);
 		return -1;
 	}
 
 	flags = GBM_BO_USE_CURSOR | GBM_BO_USE_WRITE;
 
 	for (i = 0; i < 2; i++) {
-		if (output->cursor_bo[i])
+		if (output->gbm_cursor_bo[i])
 			continue;
 
-		output->cursor_bo[i] =
+		output->gbm_cursor_bo[i] =
 			gbm_bo_create(b->gbm, b->cursor_width, b->cursor_height,
 				GBM_FORMAT_ARGB8888, flags);
 	}
 
-	if (output->cursor_bo[0] == NULL || output->cursor_bo[1] == NULL) {
+	if (output->gbm_cursor_bo[0] == NULL || output->gbm_cursor_bo[1] == NULL) {
 		weston_log("cursor buffers unavailable, using gl cursors\n");
 		b->cursors_are_broken = 1;
 	}
@@ -1933,6 +1944,9 @@ edid_parse_string(const uint8_t *data, char text[])
 	 * terminated or not junk. */
 	strncpy(text, (const char *) data, 12);
 
+	/* guarantee our new string is null-terminated */
+	text[12] = '\0';
+
 	/* remove insane chars */
 	for (i = 0; text[i] != '\0'; i++) {
 		if (text[i] == '\n' ||
@@ -2102,6 +2116,9 @@ parse_modeline(const char *s, drmModeModeInfo *mode)
 	else
 		return -1;
 
+	snprintf(mode->name, sizeof mode->name, "%dx%d@%.3f",
+		 mode->hdisplay, mode->vdisplay, fclock);
+
 	return 0;
 }
 
@@ -2129,31 +2146,23 @@ setup_output_seat_constraint(struct drm_backend *b,
 }
 
 static int
-get_gbm_format_from_section(struct weston_config_section *section,
-			    uint32_t default_value,
-			    uint32_t *format)
+parse_gbm_format(const char *s, uint32_t default_value, uint32_t *gbm_format)
 {
-	char *s;
 	int ret = 0;
 
-	weston_config_section_get_string(section,
-					 "gbm-format", &s, NULL);
-
 	if (s == NULL)
-		*format = default_value;
+		*gbm_format = default_value;
 	else if (strcmp(s, "xrgb8888") == 0)
-		*format = GBM_FORMAT_XRGB8888;
+		*gbm_format = GBM_FORMAT_XRGB8888;
 	else if (strcmp(s, "rgb565") == 0)
-		*format = GBM_FORMAT_RGB565;
+		*gbm_format = GBM_FORMAT_RGB565;
 	else if (strcmp(s, "xrgb2101010") == 0)
-		*format = GBM_FORMAT_XRGB2101010;
+		*gbm_format = GBM_FORMAT_XRGB2101010;
 	else {
 		weston_log("fatal: unrecognized pixel format: %s\n", s);
 		ret = -1;
 	}
 
-	free(s);
-
 	return ret;
 }
 
@@ -2172,25 +2181,42 @@ get_gbm_format_from_section(struct weston_config_section *section,
  * @returns A mode from the output's mode list, or NULL if none available
  */
 static struct drm_mode *
-drm_output_choose_initial_mode(struct drm_output *output,
-			       enum output_config kind,
-			       int width, int height,
-			       const drmModeModeInfo *current_mode,
-			       const drmModeModeInfo *modeline)
+drm_output_choose_initial_mode(struct drm_backend *backend,
+			       struct drm_output *output,
+			       enum weston_drm_backend_output_mode mode,
+			       struct weston_drm_backend_output_config *config,
+			       const drmModeModeInfo *current_mode)
 {
 	struct drm_mode *preferred = NULL;
 	struct drm_mode *current = NULL;
 	struct drm_mode *configured = NULL;
 	struct drm_mode *best = NULL;
 	struct drm_mode *drm_mode;
+	drmModeModeInfo modeline;
+	int32_t width = 0;
+	int32_t height = 0;
+
+	if (mode == WESTON_DRM_BACKEND_OUTPUT_PREFERRED && config->modeline) {
+		if (sscanf(config->modeline, "%dx%d", &width, &height) != 2) {
+			width = -1;
+
+			if (parse_modeline(config->modeline, &modeline) == 0) {
+				configured = drm_output_add_mode(output, &modeline);
+				if (!configured)
+					return NULL;
+			} else {
+				weston_log("Invalid modeline \"%s\" for output %s\n",
+					   config->modeline, output->base.name);
+			}
+		}
+	}
 
 	wl_list_for_each_reverse(drm_mode, &output->base.mode_list, base.link) {
-		if (kind == OUTPUT_CONFIG_MODE &&
-		    width == drm_mode->base.width &&
+		if (width == drm_mode->base.width &&
 		    height == drm_mode->base.height)
 			configured = drm_mode;
 
-		if (memcmp(&current_mode, &drm_mode->mode_info,
+		if (memcmp(current_mode, &drm_mode->mode_info,
 			   sizeof *current_mode) == 0)
 			current = drm_mode;
 
@@ -2200,24 +2226,15 @@ drm_output_choose_initial_mode(struct drm_output *output,
 		best = drm_mode;
 	}
 
-	if (kind == OUTPUT_CONFIG_MODELINE) {
-		configured = drm_output_add_mode(output, modeline);
-		if (!configured)
-			return NULL;
-	}
-
 	if (current == NULL && current_mode->clock != 0) {
 		current = drm_output_add_mode(output, current_mode);
 		if (!current)
 			return NULL;
 	}
 
-	if (kind == OUTPUT_CONFIG_CURRENT)
+	if (mode == WESTON_DRM_BACKEND_OUTPUT_CURRENT)
 		configured = current;
 
-	if (option_current_mode && current)
-		return current;
-
 	if (configured)
 		return configured;
 
@@ -2281,12 +2298,11 @@ create_output_for_connector(struct drm_backend *b,
 	struct drm_output *output;
 	struct drm_mode *drm_mode, *next, *current;
 	struct weston_mode *m;
-	struct weston_config_section *section;
-	drmModeModeInfo crtc_mode, modeline;
-	int i, width, height, scale;
-	char *s;
-	enum output_config config;
-	uint32_t transform;
+
+	drmModeModeInfo crtc_mode;
+	int i;
+	enum weston_drm_backend_output_mode mode;
+	struct weston_drm_backend_output_config config = {{ 0 }};
 
 	i = find_crtc_for_connector(b, resources, connector);
 	if (i < 0) {
@@ -2305,42 +2321,14 @@ create_output_for_connector(struct drm_backend *b,
 	output->base.serial_number = "unknown";
 	wl_list_init(&output->base.mode_list);
 
-	section = weston_config_get_section(b->compositor->config, "output", "name",
-					    output->base.name);
-	weston_config_section_get_string(section, "mode", &s, "preferred");
-	if (strcmp(s, "off") == 0)
-		config = OUTPUT_CONFIG_OFF;
-	else if (strcmp(s, "preferred") == 0)
-		config = OUTPUT_CONFIG_PREFERRED;
-	else if (strcmp(s, "current") == 0)
-		config = OUTPUT_CONFIG_CURRENT;
-	else if (sscanf(s, "%dx%d", &width, &height) == 2)
-		config = OUTPUT_CONFIG_MODE;
-	else if (parse_modeline(s, &modeline) == 0)
-		config = OUTPUT_CONFIG_MODELINE;
-	else {
-		weston_log("Invalid mode \"%s\" for output %s\n",
-			   s, output->base.name);
-		config = OUTPUT_CONFIG_PREFERRED;
-	}
-	free(s);
+	mode = b->configure_output(b->compositor, b->use_current_mode,
+				   output->base.name, &config);
+	if (parse_gbm_format(config.gbm_format, b->gbm_format, &output->gbm_format) == -1)
+		output->gbm_format = b->gbm_format;
 
-	weston_config_section_get_int(section, "scale", &scale, 1);
-	weston_config_section_get_string(section, "transform", &s, "normal");
-	if (weston_parse_transform(s, &transform) < 0)
-		weston_log("Invalid transform \"%s\" for output %s\n",
-			   s, output->base.name);
-
-	free(s);
-
-	if (get_gbm_format_from_section(section,
-					b->format,
-					&output->format) == -1)
-		output->format = b->format;
-
-	weston_config_section_get_string(section, "seat", &s, "");
-	setup_output_seat_constraint(b, &output->base, s);
-	free(s);
+	setup_output_seat_constraint(b, &output->base,
+				     config.seat ? config.seat : "");
+	free(config.seat);
 
 	output->crtc_id = resources->crtcs[i];
 	output->pipe = i;
@@ -2360,16 +2348,15 @@ create_output_for_connector(struct drm_backend *b,
 			goto err_free;
 	}
 
-	if (config == OUTPUT_CONFIG_OFF) {
+	if (mode == WESTON_DRM_BACKEND_OUTPUT_OFF) {
 		weston_log("Disabling output %s\n", output->base.name);
 		drmModeSetCrtc(b->drm.fd, output->crtc_id,
 			       0, 0, 0, 0, 0, NULL);
 		goto err_free;
 	}
 
-	current = drm_output_choose_initial_mode(output, config,
-						 width, height,
-						 &crtc_mode, &modeline);
+	current = drm_output_choose_initial_mode(b, output, mode, &config,
+						 &crtc_mode);
 	if (!current)
 		goto err_free;
 	output->base.current_mode = &current->base;
@@ -2377,7 +2364,7 @@ create_output_for_connector(struct drm_backend *b,
 
 	weston_output_init(&output->base, b->compositor, x, y,
 			   connector->mmWidth, connector->mmHeight,
-			   transform, scale);
+			   config.base.transform, config.base.scale);
 
 	if (b->use_pixman) {
 		if (drm_output_init_pixman(output, b) < 0) {
@@ -2416,7 +2403,8 @@ create_output_for_connector(struct drm_backend *b,
 	output->base.gamma_size = output->original_crtc->gamma_size;
 	output->base.set_gamma = drm_output_set_gamma;
 
-	weston_plane_init(&output->cursor_plane, b->compositor, 0, 0);
+	weston_plane_init(&output->cursor_plane, b->compositor,
+			  INT32_MIN, INT32_MIN);
 	weston_plane_init(&output->fb_plane, b->compositor, 0, 0);
 
 	weston_compositor_stack_plane(b->compositor, &output->cursor_plane, NULL);
@@ -2454,6 +2442,7 @@ err_free:
 	b->crtc_allocator &= ~(1 << output->crtc_id);
 	b->connector_allocator &= ~(1 << output->connector_id);
 	free(output);
+	free(config.modeline);
 
 	return -1;
 }
@@ -2720,7 +2709,6 @@ drm_destroy(struct weston_compositor *ec)
 	weston_launcher_destroy(ec->launcher);
 
 	close(b->drm.fd);
-
 	free(b);
 }
 
@@ -2801,15 +2789,6 @@ session_notify(struct wl_listener *listener, void *data)
 	};
 }
 
-static void
-switch_vt_binding(struct weston_keyboard *keyboard, uint32_t time,
-		  uint32_t key, void *data)
-{
-	struct weston_compositor *compositor = data;
-
-	weston_launcher_activate_vt(compositor->launcher, key - KEY_F1 + 1);
-}
-
 /*
  * Find primary GPU
  * Some systems may have multiple DRM devices attached to a single seat. This
@@ -2959,7 +2938,7 @@ recorder_binding(struct weston_keyboard *keyboard, uint32_t time, uint32_t key,
 			      struct drm_output, base.link);
 
 	if (!output->recorder) {
-		if (output->format != GBM_FORMAT_XRGB8888) {
+		if (output->gbm_format != GBM_FORMAT_XRGB8888) {
 			weston_log("failed to start vaapi recorder: "
 				   "output format not supported\n");
 			return;
@@ -3053,16 +3032,13 @@ renderer_switch_binding(struct weston_keyboard *keyboard, uint32_t time,
 
 static struct drm_backend *
 drm_backend_create(struct weston_compositor *compositor,
-		      struct drm_parameters *param,
-		      int *argc, char *argv[],
-		      struct weston_config *config)
+		   struct weston_drm_backend_config *config)
 {
 	struct drm_backend *b;
-	struct weston_config_section *section;
 	struct udev_device *drm_device;
 	struct wl_event_loop *loop;
 	const char *path;
-	uint32_t key;
+	const char *seat_id = default_seat;
 
 	weston_log("initializing drm backend\n");
 
@@ -3082,18 +3058,19 @@ drm_backend_create(struct weston_compositor *compositor,
 	 */
 	b->sprites_are_broken = 1;
 	b->compositor = compositor;
+	b->use_pixman = config->use_pixman;
+	b->configure_output = config->configure_output;
+	b->use_current_mode = config->use_current_mode;
 
-	section = weston_config_get_section(config, "core", NULL, NULL);
-	if (get_gbm_format_from_section(section,
-					GBM_FORMAT_XRGB8888,
-					&b->format) == -1)
-		goto err_base;
+	if (parse_gbm_format(config->gbm_format, GBM_FORMAT_XRGB8888, &b->gbm_format) < 0)
+		goto err_compositor;
 
-	b->use_pixman = param->use_pixman;
+	if (config->seat_id)
+		seat_id = config->seat_id;
 
 	/* Check if we run drm-backend using weston-launch */
-	compositor->launcher = weston_launcher_connect(compositor, param->tty,
-						       param->seat_id, true);
+	compositor->launcher = weston_launcher_connect(compositor, config->tty,
+						       seat_id, true);
 	if (compositor->launcher == NULL) {
 		weston_log("fatal: drm backend should be run "
 			   "using weston-launch binary or as root\n");
@@ -3109,7 +3086,7 @@ drm_backend_create(struct weston_compositor *compositor,
 	b->session_listener.notify = session_notify;
 	wl_signal_add(&compositor->session_signal, &b->session_listener);
 
-	drm_device = find_primary_gpu(b, param->seat_id);
+	drm_device = find_primary_gpu(b, seat_id);
 	if (drm_device == NULL) {
 		weston_log("no drm device found\n");
 		goto err_udev;
@@ -3138,21 +3115,18 @@ drm_backend_create(struct weston_compositor *compositor,
 
 	b->prev_state = WESTON_COMPOSITOR_ACTIVE;
 
-	for (key = KEY_F1; key < KEY_F9; key++)
-		weston_compositor_add_key_binding(compositor, key,
-						  MODIFIER_CTRL | MODIFIER_ALT,
-						  switch_vt_binding, compositor);
+	weston_setup_vt_switch_bindings(compositor);
 
 	wl_list_init(&b->sprite_list);
 	create_sprites(b);
 
 	if (udev_input_init(&b->input,
-			    compositor, b->udev, param->seat_id) < 0) {
+			    compositor, b->udev, seat_id) < 0) {
 		weston_log("failed to create input devices\n");
 		goto err_sprite;
 	}
 
-	if (create_outputs(b, param->connector, drm_device) < 0) {
+	if (create_outputs(b, config->connector, drm_device) < 0) {
 		weston_log("failed to create output for %s\n", path);
 		goto err_udev_input;
 	}
@@ -3217,7 +3191,8 @@ err_drm_source:
 err_udev_input:
 	udev_input_destroy(&b->input);
 err_sprite:
-	gbm_device_destroy(b->gbm);
+	if (b->gbm)
+		gbm_device_destroy(b->gbm);
 	destroy_sprites(b);
 err_udev_dev:
 	udev_device_unref(drm_device);
@@ -3227,32 +3202,37 @@ err_udev:
 	udev_unref(b->udev);
 err_compositor:
 	weston_compositor_shutdown(compositor);
-err_base:
 	free(b);
 	return NULL;
 }
 
+static void
+config_init_to_defaults(struct weston_drm_backend_config *config)
+{
+}
+
 WL_EXPORT int
-backend_init(struct weston_compositor *compositor, int *argc, char *argv[],
-	     struct weston_config *config)
+backend_init(struct weston_compositor *compositor,
+	     int *argc, char *argv[],
+	     struct weston_config *wc,
+	     struct weston_backend_config *config_base)
 {
 	struct drm_backend *b;
-	struct drm_parameters param = { 0, };
+	struct weston_drm_backend_config config = {{ 0, }};
 
-	const struct weston_option drm_options[] = {
-		{ WESTON_OPTION_INTEGER, "connector", 0, &param.connector },
-		{ WESTON_OPTION_STRING, "seat", 0, &param.seat_id },
-		{ WESTON_OPTION_INTEGER, "tty", 0, &param.tty },
-		{ WESTON_OPTION_BOOLEAN, "current-mode", 0, &option_current_mode },
-		{ WESTON_OPTION_BOOLEAN, "use-pixman", 0, &param.use_pixman },
-	};
+	if (config_base == NULL ||
+	    config_base->struct_version != WESTON_DRM_BACKEND_CONFIG_VERSION ||
+	    config_base->struct_size > sizeof(struct weston_drm_backend_config)) {
+		weston_log("drm backend config structure is invalid\n");
+		return -1;
+	}
 
-	param.seat_id = default_seat;
+	config_init_to_defaults(&config);
+	memcpy(&config, config_base, config_base->struct_size);
 
-	parse_options(drm_options, ARRAY_LENGTH(drm_options), argc, argv);
-
-	b = drm_backend_create(compositor, &param, argc, argv, config);
+	b = drm_backend_create(compositor, &config);
 	if (b == NULL)
 		return -1;
+
 	return 0;
 }
