@@ -44,11 +44,12 @@
 
 #include "shared/helpers.h"
 #include "compositor.h"
+#include "compositor-fbdev.h"
 #include "launcher-util.h"
 #include "pixman-renderer.h"
 #include "libinput-seat.h"
 #include "gl-renderer.h"
-#include "presentation_timing-server-protocol.h"
+#include "presentation-time-server-protocol.h"
 
 struct fbdev_backend {
 	struct weston_backend base;
@@ -58,6 +59,7 @@ struct fbdev_backend {
 	struct udev *udev;
 	struct udev_input input;
 	int use_pixman;
+	uint32_t output_transform;
 	struct wl_listener session_listener;
 };
 
@@ -84,23 +86,15 @@ struct fbdev_output {
 	struct wl_event_source *finish_frame_timer;
 
 	/* Frame buffer details. */
-	const char *device; /* ownership shared with fbdev_parameters */
+	char *device;
 	struct fbdev_screeninfo fb_info;
 	void *fb; /* length is fb_info.buffer_length */
 
 	/* pixman details. */
 	pixman_image_t *hw_surface;
-	pixman_image_t *shadow_surface;
-	void *shadow_buf;
 	uint8_t depth;
 };
 
-struct fbdev_parameters {
-	int tty;
-	char *device;
-	int use_gl;
-};
-
 struct gl_renderer_interface *gl_renderer;
 
 static const char default_seat[] = "seat0";
@@ -123,7 +117,7 @@ fbdev_output_start_repaint_loop(struct weston_output *output)
 	struct timespec ts;
 
 	weston_compositor_read_presentation_clock(output->compositor, &ts);
-	weston_output_finish_frame(output, &ts, PRESENTATION_FEEDBACK_INVALID);
+	weston_output_finish_frame(output, &ts, WP_PRESENTATION_FEEDBACK_INVALID);
 }
 
 static void
@@ -131,39 +125,11 @@ fbdev_output_repaint_pixman(struct weston_output *base, pixman_region32_t *damag
 {
 	struct fbdev_output *output = to_fbdev_output(base);
 	struct weston_compositor *ec = output->base.compositor;
-	pixman_box32_t *rects;
-	int nrects, i;
 
 	/* Repaint the damaged region onto the back buffer. */
-	pixman_renderer_output_set_buffer(base, output->shadow_surface);
+	pixman_renderer_output_set_buffer(base, output->hw_surface);
 	ec->renderer->repaint_output(base, damage);
 
-	/* Transform and composite onto the frame buffer. */
-	rects = pixman_region32_rectangles(damage, &nrects);
-
-	for (i = 0; i < nrects; i++) {
-		pixman_box32_t transformed_rect;
-		int width, height;
-
-		transformed_rect = weston_transformed_rect(base->width,
-							   base->height,
-							   base->transform,
-							   1, rects[i]);
-		width = transformed_rect.x2 - transformed_rect.x1;
-		height = transformed_rect.y2 - transformed_rect.y1;
-		pixman_image_composite32(PIXMAN_OP_SRC,
-			output->shadow_surface, /* src */
-			NULL /* mask */,
-			output->hw_surface, /* dest */
-			transformed_rect.x1, /* src_x */
-			transformed_rect.y1, /* src_y */
-			0, 0, /* mask_x, mask_y */
-			transformed_rect.x1, /* dest_x */
-			transformed_rect.y1, /* dest_y */
-			width, /* width */
-			height /* height */);
-	}
-
 	/* Update the damage region. */
 	pixman_region32_subtract(&ec->primary_plane.damage,
 	                         &ec->primary_plane.damage, damage);
@@ -338,6 +304,7 @@ fbdev_query_screen_info(struct fbdev_output *output, int fd,
 	info->buffer_length = fixinfo.smem_len;
 	info->line_length = fixinfo.line_length;
 	strncpy(info->id, fixinfo.id, sizeof(info->id));
+	info->id[sizeof(info->id)-1] = '\0';
 
 	info->pixel_format = calculate_pixman_format(&varinfo, &fixinfo);
 	info->refresh_rate = calculate_refresh_rate(&varinfo);
@@ -486,13 +453,8 @@ fbdev_output_create(struct fbdev_backend *backend,
                     const char *device)
 {
 	struct fbdev_output *output;
-	struct weston_config_section *section;
 	int fb_fd;
-	int width, height;
-	unsigned int bytes_per_pixel;
 	struct wl_event_loop *loop;
-	uint32_t config_transform;
-	char *s;
 
 	weston_log("Creating fbdev output.\n");
 
@@ -501,7 +463,7 @@ fbdev_output_create(struct fbdev_backend *backend,
 		return -1;
 
 	output->backend = backend;
-	output->device = device;
+	output->device = strdup(device);
 
 	/* Create the frame buffer. */
 	fb_fd = fbdev_frame_buffer_open(output, device, &output->fb_info);
@@ -537,39 +499,15 @@ fbdev_output_create(struct fbdev_backend *backend,
 	output->base.model = output->fb_info.id;
 	output->base.name = strdup("fbdev");
 
-	section = weston_config_get_section(backend->compositor->config,
-					    "output", "name",
-					    output->base.name);
-	weston_config_section_get_string(section, "transform", &s, "normal");
-	if (weston_parse_transform(s, &config_transform) < 0)
-		weston_log("Invalid transform \"%s\" for output %s\n",
-			   s, output->base.name);
-	free(s);
-
 	weston_output_init(&output->base, backend->compositor,
 	                   0, 0, output->fb_info.width_mm,
 	                   output->fb_info.height_mm,
-	                   config_transform,
+	                   backend->output_transform,
 			   1);
 
-	width = output->mode.width;
-	height = output->mode.height;
-	bytes_per_pixel = output->fb_info.bits_per_pixel / 8;
-
-	output->shadow_buf = malloc(width * height * bytes_per_pixel);
-	output->shadow_surface =
-		pixman_image_create_bits(output->fb_info.pixel_format,
-		                         width, height,
-		                         output->shadow_buf,
-		                         width * bytes_per_pixel);
-	if (output->shadow_buf == NULL || output->shadow_surface == NULL) {
-		weston_log("Failed to create surface for frame buffer.\n");
-		goto out_hw_surface;
-	}
-
 	if (backend->use_pixman) {
 		if (pixman_renderer_output_create(&output->base) < 0)
-			goto out_shadow_surface;
+			goto out_hw_surface;
 	} else {
 		setenv("HYBRIS_EGLPLATFORM", "wayland", 1);
 		if (gl_renderer->output_create(&output->base,
@@ -577,11 +515,10 @@ fbdev_output_create(struct fbdev_backend *backend,
 					       gl_renderer->opaque_attribs,
 					       NULL, 0) < 0) {
 			weston_log("gl_renderer_output_create failed.\n");
-			goto out_shadow_surface;
+			goto out_hw_surface;
 		}
 	}
 
-
 	loop = wl_display_get_event_loop(backend->compositor->wl_display);
 	output->finish_frame_timer =
 		wl_event_loop_add_timer(loop, finish_frame_handler, output);
@@ -595,16 +532,13 @@ fbdev_output_create(struct fbdev_backend *backend,
 
 	return 0;
 
-out_shadow_surface:
-	pixman_image_unref(output->shadow_surface);
-	output->shadow_surface = NULL;
 out_hw_surface:
-	free(output->shadow_buf);
 	pixman_image_unref(output->hw_surface);
 	output->hw_surface = NULL;
 	weston_output_destroy(&output->base);
 	fbdev_frame_buffer_destroy(output);
 out_free:
+	free(output->device);
 	free(output);
 
 	return -1;
@@ -624,16 +558,6 @@ fbdev_output_destroy(struct weston_output *base)
 	if (backend->use_pixman) {
 		if (base->renderer_state != NULL)
 			pixman_renderer_output_destroy(base);
-
-		if (output->shadow_surface != NULL) {
-			pixman_image_unref(output->shadow_surface);
-			output->shadow_surface = NULL;
-		}
-
-		if (output->shadow_buf != NULL) {
-			free(output->shadow_buf);
-			output->shadow_buf = NULL;
-		}
 	} else {
 		gl_renderer->output_destroy(base);
 	}
@@ -641,6 +565,7 @@ fbdev_output_destroy(struct weston_output *base)
 	/* Remove the output. */
 	weston_output_destroy(&output->base);
 
+	free(output->device);
 	free(output);
 }
 
@@ -668,7 +593,7 @@ fbdev_output_reenable(struct fbdev_backend *backend,
 	struct fbdev_output *output = to_fbdev_output(base);
 	struct fbdev_screeninfo new_screen_info;
 	int fb_fd;
-	const char *device;
+	char *device;
 
 	weston_log("Re-enabling fbdev output.\n");
 
@@ -695,9 +620,10 @@ fbdev_output_reenable(struct fbdev_backend *backend,
 		/* Remove and re-add the output so that resources depending on
 		 * the frame buffer X/Y resolution (such as the shadow buffer)
 		 * are re-initialised. */
-		device = output->device;
-		fbdev_output_destroy(base);
+		device = strdup(output->device);
+		fbdev_output_destroy(&output->base);
 		fbdev_output_create(backend, device);
+		free(device);
 
 		return 0;
 	}
@@ -785,7 +711,7 @@ session_notify(struct wl_listener *listener, void *data)
 		/* If we have a repaint scheduled (from the idle handler), make
 		 * sure we cancel that so we don't try to pageflip when we're
 		 * vt switched away.  The OFFSCREEN state will prevent
-		 * further attemps at repainting.  When we switch
+		 * further attempts at repainting.  When we switch
 		 * back, we schedule a repaint, which will process
 		 * pending frame callbacks. */
 
@@ -802,23 +728,13 @@ fbdev_restore(struct weston_compositor *compositor)
 	weston_launcher_restore(compositor->launcher);
 }
 
-static void
-switch_vt_binding(struct weston_keyboard *keyboard, uint32_t time,
-		  uint32_t key, void *data)
-{
-	struct weston_compositor *compositor = data;
-
-	weston_launcher_activate_vt(compositor->launcher, key - KEY_F1 + 1);
-}
-
 static struct fbdev_backend *
 fbdev_backend_create(struct weston_compositor *compositor, int *argc, char *argv[],
                      struct weston_config *config,
-                     struct fbdev_parameters *param)
+                     struct weston_fbdev_backend_config *param)
 {
 	struct fbdev_backend *backend;
 	const char *seat_id = default_seat;
-	uint32_t key;
 
 	weston_log("initializing fbdev backend\n");
 
@@ -854,12 +770,10 @@ fbdev_backend_create(struct weston_compositor *compositor, int *argc, char *argv
 
 	backend->prev_state = WESTON_COMPOSITOR_ACTIVE;
 	backend->use_pixman = !param->use_gl;
+	backend->output_transform = param->output_transform;
+
+	weston_setup_vt_switch_bindings(compositor);
 
-	for (key = KEY_F1; key < KEY_F9; key++)
-		weston_compositor_add_key_binding(compositor, key,
-		                                  MODIFIER_CTRL | MODIFIER_ALT,
-		                                  switch_vt_binding,
-		                                  compositor);
 	if (backend->use_pixman) {
 		if (pixman_renderer_init(compositor) < 0)
 			goto out_launcher;
@@ -901,28 +815,36 @@ out_compositor:
 	return NULL;
 }
 
-WL_EXPORT int
-backend_init(struct weston_compositor *compositor, int *argc, char *argv[],
-	     struct weston_config *config)
+static void
+config_init_to_defaults(struct weston_fbdev_backend_config *config)
 {
-	struct fbdev_backend *b;
 	/* TODO: Ideally, available frame buffers should be enumerated using
 	 * udev, rather than passing a device node in as a parameter. */
-	struct fbdev_parameters param = {
-		.tty = 0, /* default to current tty */
-		.device = "/dev/fb0", /* default frame buffer */
-		.use_gl = 0,
-	};
+	config->tty = 0; /* default to current tty */
+	config->device = "/dev/fb0"; /* default frame buffer */
+	config->use_gl = 0;
+	config->output_transform = WL_OUTPUT_TRANSFORM_NORMAL;
+}
 
-	const struct weston_option fbdev_options[] = {
-		{ WESTON_OPTION_INTEGER, "tty", 0, &param.tty },
-		{ WESTON_OPTION_STRING, "device", 0, &param.device },
-		{ WESTON_OPTION_BOOLEAN, "use-gl", 0, &param.use_gl },
-	};
+WL_EXPORT int
+backend_init(struct weston_compositor *compositor, int *argc, char *argv[],
+	     struct weston_config *wc,
+	     struct weston_backend_config *config_base)
+{
+	struct fbdev_backend *b;
+	struct weston_fbdev_backend_config config = {{ 0, }};
 
-	parse_options(fbdev_options, ARRAY_LENGTH(fbdev_options), argc, argv);
+	if (config_base == NULL ||
+	    config_base->struct_version != WESTON_FBDEV_BACKEND_CONFIG_VERSION ||
+	    config_base->struct_size > sizeof(struct weston_fbdev_backend_config)) {
+		weston_log("fbdev backend config structure is invalid\n");
+		return -1;
+	}
 
-	b = fbdev_backend_create(compositor, argc, argv, config, &param);
+	config_init_to_defaults(&config);
+	memcpy(&config, config_base, config_base->struct_size);
+
+	b = fbdev_backend_create(compositor, argc, argv, wc, &config);
 	if (b == NULL)
 		return -1;
 	return 0;
