@@ -42,7 +42,7 @@
 #include "compositor.h"
 #include "shared/helpers.h"
 #include "shared/os-compatibility.h"
-#include "fullscreen-shell-client-protocol.h"
+#include "fullscreen-shell-unstable-v1-client-protocol.h"
 
 struct shared_output {
 	struct weston_output *output;
@@ -55,11 +55,11 @@ struct shared_output {
 		struct wl_compositor *compositor;
 		struct wl_shm *shm;
 		uint32_t shm_formats;
-		struct _wl_fullscreen_shell *fshell;
+		struct zwp_fullscreen_shell_v1 *fshell;
 		struct wl_output *output;
 		struct wl_surface *surface;
 		struct wl_callback *frame_cb;
-		struct _wl_fullscreen_shell_mode_feedback *mode_feedback;
+		struct zwp_fullscreen_shell_mode_feedback_v1 *mode_feedback;
 	} parent;
 
 	struct wl_event_source *event_source;
@@ -142,7 +142,9 @@ ss_seat_handle_motion(void *data, struct wl_pointer *pointer,
 	/* No transformation of input position is required here because we are
 	 * always receiving the input in the same coordinates as the output. */
 
-	notify_motion_absolute(&seat->base, time, x, y);
+	notify_motion_absolute(&seat->base, time,
+			       wl_fixed_to_double(x), wl_fixed_to_double(y));
+	notify_pointer_frame(&seat->base);
 }
 
 static void
@@ -153,6 +155,7 @@ ss_seat_handle_button(void *data, struct wl_pointer *pointer,
 	struct ss_seat *seat = data;
 
 	notify_button(&seat->base, time, button, state);
+	notify_pointer_frame(&seat->base);
 }
 
 static void
@@ -160,8 +163,14 @@ ss_seat_handle_axis(void *data, struct wl_pointer *pointer,
 		    uint32_t time, uint32_t axis, wl_fixed_t value)
 {
 	struct ss_seat *seat = data;
+	struct weston_pointer_axis_event weston_event;
 
-	notify_axis(&seat->base, time, axis, value);
+	weston_event.axis = axis;
+	weston_event.value = wl_fixed_to_double(value);
+	weston_event.has_discrete = false;
+
+	notify_axis(&seat->base, time, &weston_event);
+	notify_pointer_frame(&seat->base);
 }
 
 static const struct wl_pointer_listener ss_seat_pointer_listener = {
@@ -438,13 +447,13 @@ shared_output_get_shm_buffer(struct shared_output *so)
 
 	fd = os_create_anonymous_file(height * stride);
 	if (fd < 0) {
-		weston_log("os_create_anonymous_file: %m");
+		weston_log("os_create_anonymous_file: %m\n");
 		return NULL;
 	}
 
 	data = mmap(NULL, height * stride, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
 	if (data == MAP_FAILED) {
-		weston_log("mmap: %m");
+		weston_log("mmap: %m\n");
 		goto out_close;
 	}
 
@@ -697,10 +706,12 @@ registry_handle_global(void *data, struct wl_registry *registry,
 			wl_registry_bind(registry,
 					 id, &wl_shm_interface, 1);
 		wl_shm_add_listener(so->parent.shm, &shm_listener, so);
-	} else if (strcmp(interface, "_wl_fullscreen_shell") == 0) {
+	} else if (strcmp(interface, "zwp_fullscreen_shell_v1") == 0) {
 		so->parent.fshell =
 			wl_registry_bind(registry,
-					 id, &_wl_fullscreen_shell_interface, 1);
+					 id,
+					 &zwp_fullscreen_shell_v1_interface,
+					 1);
 	}
 }
 
@@ -750,25 +761,25 @@ output_destroyed(struct wl_listener *l, void *data)
 }
 
 static void
-mode_feedback_ok(void *data, struct _wl_fullscreen_shell_mode_feedback *fb)
+mode_feedback_ok(void *data, struct zwp_fullscreen_shell_mode_feedback_v1 *fb)
 {
 	struct shared_output *so = data;
 
-	_wl_fullscreen_shell_mode_feedback_destroy(so->parent.mode_feedback);
+	zwp_fullscreen_shell_mode_feedback_v1_destroy(so->parent.mode_feedback);
 }
 
 static void
-mode_feedback_failed(void *data, struct _wl_fullscreen_shell_mode_feedback *fb)
+mode_feedback_failed(void *data, struct zwp_fullscreen_shell_mode_feedback_v1 *fb)
 {
 	struct shared_output *so = data;
 
-	_wl_fullscreen_shell_mode_feedback_destroy(so->parent.mode_feedback);
+	zwp_fullscreen_shell_mode_feedback_v1_destroy(so->parent.mode_feedback);
 
 	weston_log("Screen share failed: present_surface_for_mode failed\n");
 	shared_output_destroy(so);
 }
 
-struct _wl_fullscreen_shell_mode_feedback_listener mode_feedback_listener = {
+struct zwp_fullscreen_shell_mode_feedback_v1_listener mode_feedback_listener = {
 	mode_feedback_ok,
 	mode_feedback_failed,
 	mode_feedback_ok,
@@ -914,22 +925,22 @@ shared_output_create(struct weston_output *output, int parent_fd)
 	so->parent.surface =
 		wl_compositor_create_surface(so->parent.compositor);
 	if (!so->parent.surface) {
-		weston_log("Screen share failed: %m");
+		weston_log("Screen share failed: %m\n");
 		goto err_display;
 	}
 
 	so->parent.mode_feedback =
-		_wl_fullscreen_shell_present_surface_for_mode(so->parent.fshell,
-							      so->parent.surface,
-							      so->parent.output,
-							      output->current_mode->refresh);
+		zwp_fullscreen_shell_v1_present_surface_for_mode(so->parent.fshell,
+								 so->parent.surface,
+								 so->parent.output,
+								 output->current_mode->refresh);
 	if (!so->parent.mode_feedback) {
-		weston_log("Screen share failed: %m");
+		weston_log("Screen share failed: %m\n");
 		goto err_display;
 	}
-	_wl_fullscreen_shell_mode_feedback_add_listener(so->parent.mode_feedback,
-						        &mode_feedback_listener,
-						        so);
+	zwp_fullscreen_shell_mode_feedback_v1_add_listener(so->parent.mode_feedback,
+							   &mode_feedback_listener,
+							   so);
 
 	loop = wl_display_get_event_loop(output->compositor->wl_display);
 
@@ -938,7 +949,7 @@ shared_output_create(struct weston_output *output, int parent_fd)
 		wl_event_loop_add_fd(loop, epoll_fd, WL_EVENT_READABLE,
 				     shared_output_handle_event, so);
 	if (!so->event_source) {
-		weston_log("Screen share failed: %m");
+		weston_log("Screen share failed: %m\n");
 		goto err_display;
 	}
 
