@@ -29,6 +29,7 @@
 #include <string.h>
 
 #include "shared/helpers.h"
+#include "shared/xalloc.h"
 #include "weston-test-client-helper.h"
 #include "ivi-application-client-protocol.h"
 #include "ivi-test.h"
@@ -204,8 +205,6 @@ const char * const surface_property_commit_changes_test_names[] = {
 	"commit_changes_after_visibility_set_surface_destroy",
 	"commit_changes_after_opacity_set_surface_destroy",
 	"commit_changes_after_orientation_set_surface_destroy",
-	"commit_changes_after_dimension_set_surface_destroy",
-	"commit_changes_after_position_set_surface_destroy",
 	"commit_changes_after_source_rectangle_set_surface_destroy",
 	"commit_changes_after_destination_rectangle_set_surface_destroy",
 };
