@@ -1,23 +1,26 @@
 /*
  * Copyright © 2012 Intel Corporation
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting documentation, and
- * that the name of the copyright holders not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  The copyright holders make no representations
- * about the suitability of this software for any purpose.  It is provided "as
- * is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
- * OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the
+ * next paragraph) shall be included in all copies or substantial
+ * portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+ * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+ * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include "config.h"
@@ -28,25 +31,17 @@
 #include <unistd.h>
 #include <errno.h>
 #include <sys/mman.h>
+#include <cairo.h>
 
-#include "../shared/os-compatibility.h"
+#include "shared/os-compatibility.h"
+#include "shared/xalloc.h"
+#include "shared/zalloc.h"
 #include "weston-test-client-helper.h"
 
 #define max(a, b) (((a) > (b)) ? (a) : (b))
 #define min(a, b) (((a) > (b)) ? (b) : (a))
 #define clip(x, a, b)  min(max(x, a), b)
 
-void *
-fail_on_null(void *p)
-{
-	if (p == NULL) {
-		fprintf(stderr, "out of memory\n");
-		exit(EXIT_FAILURE);
-	}
-	return p;
-}
-
-
 int
 surface_contains(struct surface *surface, int x, int y)
 {
@@ -137,7 +132,11 @@ pointer_handle_enter(void *data, struct wl_pointer *wl_pointer,
 {
 	struct pointer *pointer = data;
 
-	pointer->focus = wl_surface_get_user_data(wl_surface);
+	if (wl_surface)
+		pointer->focus = wl_surface_get_user_data(wl_surface);
+	else
+		pointer->focus = NULL;
+
 	pointer->x = wl_fixed_to_int(x);
 	pointer->y = wl_fixed_to_int(y);
 
@@ -154,7 +153,7 @@ pointer_handle_leave(void *data, struct wl_pointer *wl_pointer,
 	pointer->focus = NULL;
 
 	fprintf(stderr, "test-client: got pointer leave, surface %p\n",
-		wl_surface_get_user_data(wl_surface));
+		wl_surface ? wl_surface_get_user_data(wl_surface) : NULL);
 }
 
 static void
@@ -192,12 +191,44 @@ pointer_handle_axis(void *data, struct wl_pointer *wl_pointer,
 		axis, wl_fixed_to_double(value));
 }
 
+static void
+pointer_handle_frame(void *data, struct wl_pointer *wl_pointer)
+{
+	fprintf(stderr, "test-client: got pointer frame\n");
+}
+
+static void
+pointer_handle_axis_source(void *data, struct wl_pointer *wl_pointer,
+			     uint32_t source)
+{
+	fprintf(stderr, "test-client: got pointer axis source %u\n", source);
+}
+
+static void
+pointer_handle_axis_stop(void *data, struct wl_pointer *wl_pointer,
+			 uint32_t time, uint32_t axis)
+{
+	fprintf(stderr, "test-client: got pointer axis stop\n");
+}
+
+static void
+pointer_handle_axis_discrete(void *data, struct wl_pointer *wl_pointer,
+			     uint32_t axis, int32_t value)
+{
+	fprintf(stderr, "test-client: got pointer axis discrete %u %d\n",
+		axis, value);
+}
+
 static const struct wl_pointer_listener pointer_listener = {
 	pointer_handle_enter,
 	pointer_handle_leave,
 	pointer_handle_motion,
 	pointer_handle_button,
 	pointer_handle_axis,
+	pointer_handle_frame,
+	pointer_handle_axis_source,
+	pointer_handle_axis_stop,
+	pointer_handle_axis_discrete,
 };
 
 static void
@@ -216,7 +247,10 @@ keyboard_handle_enter(void *data, struct wl_keyboard *wl_keyboard,
 {
 	struct keyboard *keyboard = data;
 
-	keyboard->focus = wl_surface_get_user_data(wl_surface);
+	if (wl_surface)
+		keyboard->focus = wl_surface_get_user_data(wl_surface);
+	else
+		keyboard->focus = NULL;
 
 	fprintf(stderr, "test-client: got keyboard enter, surface %p\n",
 		keyboard->focus);
@@ -231,7 +265,7 @@ keyboard_handle_leave(void *data, struct wl_keyboard *wl_keyboard,
 	keyboard->focus = NULL;
 
 	fprintf(stderr, "test-client: got keyboard leave, surface %p\n",
-		wl_surface_get_user_data(wl_surface));
+		wl_surface ? wl_surface_get_user_data(wl_surface) : NULL);
 }
 
 static void
@@ -522,7 +556,7 @@ seat_handle_capabilities(void *data, struct wl_seat *seat,
 	/* we will create/update the devices only with the right (test) seat.
 	 * If we haven't discovered which seat is the test seat, just
 	 * store capabilities and bail out */
-	if(input->seat_name && strcmp(input->seat_name, "test-seat") == 0)
+	if (input->seat_name && strcmp(input->seat_name, "test-seat") == 0)
 		input_update_devices(input);
 
 	fprintf(stderr, "test-client: got seat %p capabilities: %x\n",
@@ -976,3 +1010,159 @@ check_surfaces_match_in_clip(const struct surface *a, const struct surface *b, c
 
 	return true;
 }
+
+/** write_surface_as_png()
+ *
+ * Writes out a given weston test surface to disk as a PNG image
+ * using the provided filename (with path).
+ *
+ * @returns true if successfully saved file; false otherwise.
+ */
+bool
+write_surface_as_png(const struct surface *weston_surface, const char *fname)
+{
+	cairo_surface_t *cairo_surface;
+	cairo_status_t status;
+	int bpp = 4; /* Assume ARGB */
+	int stride = bpp * weston_surface->width;
+
+	cairo_surface = cairo_image_surface_create_for_data(weston_surface->data,
+							    CAIRO_FORMAT_ARGB32,
+							    weston_surface->width,
+							    weston_surface->height,
+							    stride);
+	printf("Writing PNG to disk\n");
+	status = cairo_surface_write_to_png(cairo_surface, fname);
+	if (status != CAIRO_STATUS_SUCCESS) {
+		printf("Failed to save screenshot: %s\n",
+		       cairo_status_to_string(status));
+		return false;
+	}
+	cairo_surface_destroy(cairo_surface);
+	return true;
+}
+
+/** load_surface_from_png()
+ *
+ * Reads a PNG image from disk using the given filename (and path)
+ * and returns as a freshly allocated weston test surface.
+ *
+ * @returns weston test surface with image, which should be free'd
+ * when no longer used; or, NULL in case of error.
+ */
+struct surface *
+load_surface_from_png(const char *fname)
+{
+	struct surface *reference;
+	cairo_surface_t *reference_cairo_surface;
+	cairo_status_t status;
+	size_t source_data_size;
+	int bpp;
+	int stride;
+
+	reference_cairo_surface = cairo_image_surface_create_from_png(fname);
+	status = cairo_surface_status(reference_cairo_surface);
+	if (status != CAIRO_STATUS_SUCCESS) {
+		printf("Could not open %s: %s\n", fname, cairo_status_to_string(status));
+		cairo_surface_destroy(reference_cairo_surface);
+		return NULL;
+	}
+
+	/* Disguise the cairo surface in a weston test surface */
+	reference = zalloc(sizeof *reference);
+	if (reference == NULL) {
+		perror("zalloc reference");
+		cairo_surface_destroy(reference_cairo_surface);
+		return NULL;
+	}
+	reference->width = cairo_image_surface_get_width(reference_cairo_surface);
+	reference->height = cairo_image_surface_get_height(reference_cairo_surface);
+	stride = cairo_image_surface_get_stride(reference_cairo_surface);
+	source_data_size = stride * reference->height;
+
+	/* Check that the file's stride matches our assumption */
+	bpp = 4;
+	if (stride != bpp * reference->width) {
+		printf("Mismatched stride for screenshot reference image %s\n", fname);
+		cairo_surface_destroy(reference_cairo_surface);
+		free(reference);
+		return NULL;
+	}
+
+	/* Allocate new buffer for our weston reference, and copy the data from
+	   the cairo surface so we can destroy it */
+	reference->data = zalloc(source_data_size);
+	if (reference->data == NULL) {
+		perror("zalloc reference data");
+		cairo_surface_destroy(reference_cairo_surface);
+		free(reference);
+		return NULL;
+	}
+	memcpy(reference->data,
+	       cairo_image_surface_get_data(reference_cairo_surface),
+	       source_data_size);
+
+	cairo_surface_destroy(reference_cairo_surface);
+	return reference;
+}
+
+/** create_screenshot_surface()
+ *
+ *  Allocates and initializes a weston test surface for use in
+ *  storing a screenshot of the client's output.  Establishes a
+ *  shm backed wl_buffer for retrieving screenshot image data
+ *  from the server, sized to match the client's output display.
+ *
+ *  @returns stack allocated surface image, which should be
+ *  free'd when done using it.
+ */
+struct surface *
+create_screenshot_surface(struct client *client)
+{
+	struct surface *screenshot;
+	screenshot = zalloc(sizeof *screenshot);
+	if (screenshot == NULL)
+		return NULL;
+	screenshot->wl_buffer = create_shm_buffer(client,
+						  client->output->width,
+						  client->output->height,
+						  &screenshot->data);
+	screenshot->height = client->output->height;
+	screenshot->width = client->output->width;
+
+	return screenshot;
+}
+
+/** capture_screenshot_of_output()
+ *
+ * Requests a screenshot from the server of the output that the
+ * client appears on.  The image data returned from the server
+ * can be accessed from the screenshot surface's data member.
+ *
+ * @returns a new surface object, which should be free'd when no
+ * longer needed.
+ */
+struct surface *
+capture_screenshot_of_output(struct client *client)
+{
+	struct surface *screenshot;
+
+	/* Create a surface to hold the screenshot */
+	screenshot = create_screenshot_surface(client);
+
+	client->test->buffer_copy_done = 0;
+	weston_test_capture_screenshot(client->test->weston_test,
+				       client->output->wl_output,
+				       screenshot->wl_buffer);
+	while (client->test->buffer_copy_done == 0)
+		if (wl_display_dispatch(client->wl_display) < 0)
+			break;
+
+	/* FIXME: Document somewhere the orientation the screenshot is taken
+	 * and how the clip coords are interpreted, in case of scaling/transform.
+	 * If we're using read_pixels() just make sure it is documented somewhere.
+	 * Protocol docs in the XML, comparison function docs in Doxygen style.
+	 */
+
+	return screenshot;
+}
