@@ -70,7 +70,6 @@ struct weston_wm {
 	xcb_colormap_t colormap;
 	struct wl_listener create_surface_listener;
 	struct wl_listener activate_listener;
-	struct wl_listener transform_listener;
 	struct wl_listener kill_listener;
 	struct wl_list unpaired_window_list;
 
@@ -129,6 +128,7 @@ struct weston_wm {
 		xcb_atom_t		 net_wm_moveresize;
 		xcb_atom_t		 net_supporting_wm_check;
 		xcb_atom_t		 net_supported;
+		xcb_atom_t		 net_active_window;
 		xcb_atom_t		 motif_wm_hints;
 		xcb_atom_t		 clipboard;
 		xcb_atom_t		 clipboard_manager;
@@ -141,6 +141,7 @@ struct weston_wm {
 		xcb_atom_t		 compound_text;
 		xcb_atom_t		 text;
 		xcb_atom_t		 string;
+		xcb_atom_t		 window;
 		xcb_atom_t		 text_plain_utf8;
 		xcb_atom_t		 text_plain;
 		xcb_atom_t		 xdnd_selection;
